/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "cipsaMultiphaseEuler.H"
#include "fvcDdt.H"
#include "fvcSup.H"
#include "fvmDiv.H"
#include "fvcGrad.H"
#include "fvcFlux.H"
#include "fvmLaplacian.H"

// * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * * //

void Foam::solvers::cipsaMultiphaseEuler::updateHUEqn()
{
    forAll(fluid.movingPhases(), movingPhasei)
    {
        phaseModel& phase = fluid_.movingPhases()[movingPhasei];
        volVectorField& UAlpha = phase.URef();
        const volScalarField& alpha = phase;
        const volScalarField& rhoAlpha = phase.thermoRef().rho();

        HUEqns.set
        (
            phase.index(),
            (
                fvm::div(phase.alphaRhoPhi(), UAlpha)
              + phase.divDevTau(UAlpha)
              + fvm::SuSp(- fvc::div(phase.alphaRhoPhi()), UAlpha)
              + MRF.DDt(alpha*rhoAlpha, UAlpha)
            )
        );
    }

    updateVmConvectionEqn();
    stabiliseHUEqn();
    relaxUEqn();
}


void Foam::solvers::cipsaMultiphaseEuler:: stabiliseHUEqn()
{
    volScalarField nuMix
    (
        volScalarField::New
        (
            "nuMix",
            mesh, dimensionedScalar(dimViscosity, Zero)
        )
    );

    volScalarField alphaMoving
    (
        volScalarField::New
        (
            "alphaMoving",
            mesh, dimensionedScalar(dimless, Zero)
        )
    );

    if (diffusiveStabilisation)
    {
        forAll(fluid.movingPhases(), movingPhasei)
        {
            phaseModel& phase = fluid_.movingPhases()[movingPhasei];
            const volScalarField& alpha = phase;

            nuMix += alpha*phase.thermo().nu();
            alphaMoving += alpha;
        }
        nuMix /= alphaMoving;
    }

    forAll(fluid.movingPhases(), movingPhasei)
    {
        phaseModel& phase = fluid_.movingPhases()[movingPhasei];
        const volScalarField& alpha = phase;
        const volScalarField& rhoAlpha = phase.rho();
        volVectorField& UAlpha = phase.URef();

        // Stabilise due to zero flux
        const dimensionedScalar residualDivU
        (
            dimensionSet(1, -3, -1, 0, 0),
            small
        );

        HUEqns[phase.index()] +=
            fvm::Sp
            (
                max
                (
                    residualDivU - fvc::div(phase.alphaRhoPhi()),
                    dimensionedScalar(dimensionSet(1, -3, -1, 0, 0), Zero)
                ),
                UAlpha
            )
          - fvc::Sp
            (
                max
                (
                    residualDivU - fvc::div(phase.alphaRhoPhi()),
                    dimensionedScalar(dimensionSet(1, -3, -1, 0, 0), Zero)
                ),
                UAlpha
            );

        // Stabilise due to zero phase fraction
        HUEqns[phase.index()] +=
            fvm::ddt
            (
                max(phase.residualAlpha() - alpha, scalar(0))*rhoAlpha,
                UAlpha
            )
          - fvc::ddt
            (
                max(phase.residualAlpha() - alpha, scalar(0))*rhoAlpha,
                UAlpha
            );

        // Add diffusive term to stabilise small phase fraction states
        if (diffusiveStabilisation)
        {
            HUEqns[phase.index()] -=
                fvm::laplacian
                (
                    max(phase.residualAlpha() - alpha, scalar(0))
                   /phase.residualAlpha()*rhoAlpha*nuMix,
                    UAlpha
                );
        }

        // Improve matrix diagonal dominance
        HUEqns[phase.index()].relax(1.0);
    }
}


void Foam::solvers::cipsaMultiphaseEuler::relaxUEqn()
{
    forAll(fluid.movingPhases(), movingPhasei)
    {
        const phaseModel& phase = fluid.movingPhases()[movingPhasei];

        const word name =
            HUEqns[phase.index()].psi().select(pimple.finalIter());

        relaxU[phase.index()] = 1;

        if (fluid.mesh().solution().relaxField(name))
        {
            relaxU[phase.index()] =
                fluid.mesh().solution().fieldRelaxationFactor(name);

            if (pimple.finalIter() && relaxU[phase.index()] != 1)
            {
                Warning << "Field relaxation factor of "
                        << relaxU[phase.index()] << " for solution variable '"
                        << name << "' is not recommended." << endl << endl;
            }
        }
    }
}


// ************************************************************************* //
