/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "cipsaMultiphaseEuler.H"
#include "segregatedPhaseInterface.H"

// * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * * //

Foam::PtrList<Foam::surfaceScalarField>
Foam::solvers::cipsaMultiphaseEuler::calcForcesSourcef()
{
    PtrList<surfaceScalarField> forcesSourcef(phases.size());
    fluid.fillFields
    (
        "forcesSourcef",
        dimensionSet(1, 0, -2, 0, 0),
        forcesSourcef
    );

    // Add potential force
    // Inspired by MomentumTransferPhaseSystem::Ffs()
    forAllConstIter(phaseSystem::FpfTable, fluid.Fpfs(), FpfIter)
    {
        const surfaceScalarField& Ff(*FpfIter());
        const phaseInterface interface(fluid, FpfIter.key());

        forcesSourcef[interface.phase1().index()] -= Ff;
        forcesSourcef[interface.phase2().index()] += Ff;
    }

    // Add lift force
    // Inspired by MomentumTransferPhaseSystem::Ffs()
    forAllConstIter(phaseSystem::FlfTable, fluid.Flfs(), FlfIter)
    {
        const surfaceScalarField& Ff(*FlfIter());
        const phaseInterface interface(fluid, FlfIter.key());

        forcesSourcef[interface.phase1().index()] -= Ff;
        forcesSourcef[interface.phase2().index()] += Ff;
    }

    // Add wall lubrication force
    // Inspired by MomentumTransferPhaseSystem::Ffs()
    forAllConstIter(phaseSystem::FwfTable, fluid.Fwfs(), FwfIter)
    {
        const surfaceScalarField& Ff(*FwfIter());
        const phaseInterface interface(fluid, FwfIter.key());

        forcesSourcef[interface.phase1().index()] -= Ff;
        forcesSourcef[interface.phase2().index()] += Ff;
    }

    // Add turbulent dispersion force
    // Inspired by MomentumTransferPhaseSystem::Ffs()
    // Reformulated for more stable, symmetrical formulation
    forAllConstIter(phaseSystem::DtTable, fluid.Dts(), DtIter)
    {
        const volScalarField& D(*DtIter());
        const phaseInterface interface(fluid, DtIter.key());

        const phaseModel& phase1(interface.phase1());
        const phaseModel& phase2(interface.phase2());

        const surfaceScalarField alphaf12
        (
            max
            (
                alphafs[phase1.index()] + alphafs[phase2.index()],
                phase1.residualAlpha() + phase2.residualAlpha()
            )
        );

        const surfaceScalarField Ff
        (
            fvc::interpolate(D)
           *segregatedPhaseInterface(phase1, phase2).gradAlphaf()
           /sqr(alphaf12)*phase1.mesh().magSf()
        );

        forcesSourcef[phase1.index()] -= Ff;
        forcesSourcef[phase2.index()] += Ff;
    }

    return forcesSourcef;
}


// ************************************************************************* //
