/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "cipsaMultiphaseEuler.H"
#include "constrainHbyA.H"
#include "constrainPressure.H"
#include "findRefCell.H"
#include "fvcDiv.H"
#include "fvcSnGrad.H"
#include "fvmLaplacian.H"
#include "fvcFlux.H"
#include "fvcReconstruct.H"

// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

void Foam::solvers::cipsaMultiphaseEuler::pressureCorrector()
{
    if (pimple.flow())
    {
        // Construct pressure "diffusivity"
        surfaceScalarField rAUf
        (
            IOobject
            (
                "rAUf",
                runTime.name(),
                mesh
            ),
            mesh,
            dimensionedScalar(dimTime/dimDensity, Zero)
        );

        // rAUfs, rHAUfs, alpharAUfs, rAUf WITH field relaxation of UAlpha
        rAUfs.setSize(phases.size());
        rHAUfs.clear();
        rHAUfs.setSize(phases.size());
        PtrList<surfaceScalarField> alpharAUfs(phases.size());
        forAll(fluid.movingPhases(), movingPhasei)
        {
            const phaseModel& phase = fluid.movingPhases()[movingPhasei];

            rAUfs.set
            (
                phase.index(),
                1.0
               /(
                    1.0/max(relaxU[phase.index()], small)
                  + rHUfs[phase.index()]
                   *(
                        fvc::interpolate(ddtUEqns[phase.index()].A())
                      + fvc::interpolate(dmdtUEqns[phase.index()].A())
                      + fvc::interpolate(fvModelsUEqns[phase.index()].A())
                      + dragDiagf[phase.index()]
                      + vmDiagf[phase.index()]
                    )
                )
            );

            rHAUfs.set
            (
                phase.index(), rHUfs[phase.index()]*rAUfs[phase.index()]
            );

            // Phase diagonal coefficients
            alpharAUfs.set
            (
                phase.index(),
                rHUfs[phase.index()]*rAUfs[phase.index()]
               *(alphafs[phase.index()] + dragPCoefff[phase.index()])
            );

            // Pressure "diffusivity"
            rAUf += alphafs[phase.index()]*alpharAUfs[phase.index()];
        }

        rAUf = mag(rAUf);

        // Mass transfer rates
        PtrList<volScalarField> dmdts(fluid.dmdts());
        PtrList<volScalarField> d2mdtdps(fluid.d2mdtdps());

        // --- Pressure corrector loop
        while (pimple.correct())
        {
            rho_ = fluid.rho();

            // Correct p_rgh for consistency with p and the updated densities
            // due to updated temperature
            p_rgh = p - rho*buoyancy.gh;

            // Correct fixed-flux BCs to be consistent with the velocity BCs
            fluid_.correctBoundaryFlux();

            PtrList<surfaceScalarField> phigs(phases.size());
            {
                // Mixture density gradient times gravitational acceleration
                const surfaceScalarField ghSnGradRho
                (
                    "ghSnGradRho",
                    buoyancy.ghf*fvc::snGrad(rho)*mesh.magSf()
                );

                // phigs WITH field relaxation of UAlpha
                forAll(fluid.movingPhases(), movingPhasei)
                {
                    const phaseModel& phase =
                        fluid.movingPhases()[movingPhasei];

                    // Combined buoyancy and force fluxes
                    phigs.set
                    (
                        phase.index(),
                        alpharAUfs[phase.index()]
                       *(
                          - ghSnGradRho
                          + fluid.surfaceTension(phase)*mesh.magSf()
                        )
                      - rHUfs[phase.index()]*rAUfs[phase.index()]
                       *phase.turbulentNormalStress()*mesh.magSf()
                    );
                }
            }

            // Update explicit drag terms for PISO loop
            PtrList<volVectorField> dragSource(calcDragSource());
            PtrList<surfaceScalarField> dragSourcef(calcDragSourcef());

            // Explicit part of the time derivative of the virtual mass for
            // PISO loop
            PtrList<volVectorField> vmSource(calcVmSource());
            PtrList<surfaceScalarField> vmSourcef(calcVmSourcef());

            // Update lift, wall lubrication and turbulent dispersion forces for
            // PISO loop
            PtrList<surfaceScalarField> forcesSourcef(calcForcesSourcef());

            // Buoyancy for dispersed phases and parallel to underresolved
            // interfaces
            PtrList<volScalarField> rhoAlphaBuoyancy(calcRhoAlphaBuoyancy());
            PtrList<volVectorField> buoyancySource
            (
                calcBuoyancySource(rhoAlphaBuoyancy)
            );
            PtrList<surfaceScalarField> buoyancySourcef
            (
                calcBuoyancySourcef(rhoAlphaBuoyancy)
            );
            rhoAlphaBuoyancy.clear();

            // Construct Total predicted flux
            surfaceScalarField phiHbyA
            (
                IOobject
                (
                    "phiHbyA",
                    runTime.name(),
                    mesh,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh,
                dimensionedScalar(dimFlux, Zero)
            );

            PtrList<volVectorField> HbyAs(phases.size());
            PtrList<surfaceScalarField> phiHbyAs(phases.size());

            forAll(fluid.movingPhases(), movingPhasei)
            {
                const phaseModel& phase = fluid.movingPhases()[movingPhasei];
                const volVectorField& UAlpha = phase.U();

                const volVectorField H
                (
                    constrainH
                    (
                        HUEqns[phase.index()].H(),
                        rHUs[phase.index()],
                        UAlpha,
                        p_rgh
                    )
                );

                // Pseudo convected velocities
                HbyAs.set(phase.index(), rHUs[phase.index()]*H);

                // Predicted fluxes for each phase
                phiHbyAs.set
                (
                    phase.index(),
                    constrainPhiHbyA
                    (
                        rAUfs[phase.index()]
                       *(
                            fvc::flux(HbyAs[phase.index()])
                          + (1 - relaxU[phase.index()])
                           /max(relaxU[phase.index()], small)
                           *MRF.absolute
                            (
                                phases[phase.index()].phi()().prevIter()
                            )
                          + rHUfs[phase.index()]
                           *(
                                ddtPhiCoeffs[phase.index()]
                              + dmdtPhiCoeffs[phase.index()]
                              + fvc::flux(fvModelsUEqns[phase.index()].H())
                              + forcesSourcef[phase.index()]
                              + buoyancySourcef[phase.index()]
                            )
                          + dragSourcef[phase.index()]
                          + vmSourcef[phase.index()]
                        ),
                        UAlpha,
                        p_rgh
                    )
                  + phigs[phase.index()]
                );

                phiHbyA += alphafs[phase.index()]*phiHbyAs[phase.index()];
            }

            MRF.makeRelative(phiHbyA);

            // Update the fixedFluxPressure BCs to ensure flux consistency
            {
                surfaceScalarField::Boundary phib
                (
                    surfaceScalarField::Internal::null(),
                    phi.boundaryField()
                );
                phib = 0;

                forAll(phases, phasei)
                {
                    const phaseModel& phase = phases[phasei];
                    phib +=
                        alphafs[phasei].boundaryField()
                       *phase.phi()().boundaryField();
                }

                setSnGrad<fixedFluxPressureFvPatchScalarField>
                (
                    p_rgh.boundaryFieldRef(),
                    (
                        phiHbyA.boundaryField() - phib
                    )
                   /(mesh.magSf().boundaryField()*rAUf.boundaryField())
                );
            }

            // Compressible pressure equations
            PtrList<fvScalarMatrix> pEqnComps
            (
                compressibilityEqns(dmdts, d2mdtdps)
            );

            // Cache p prior to solve for density update
            volScalarField p_rgh_0(p_rgh);

            // Iterate over the pressure equation to correct for
            // non-orthogonality
            while (pimple.correctNonOrthogonal())
            {
                // Construct the transport part of the pressure equation
                fvScalarMatrix pEqnIncomp
                (
                    fvc::div(phiHbyA)
                  - fvm::laplacian(rAUf, p_rgh)
                );

                // Solve
                {
                    fvScalarMatrix pEqn(pEqnIncomp);

                    forAll(phases, phasei)
                    {
                        pEqn += pEqnComps[phasei];
                    }

                    if (fluid.incompressible())
                    {
                            pEqn.setReference
                            (
                                pressureReference.refCell(),
                                pressureReference.refValue()
                            );
                    }

                    pEqn.solve();
                }

                // Correct fluxes and velocities on last non-orthogonal
                // iteration
                if (pimple.finalNonOrthogonalIter())
                {
                    phi_ = phiHbyA + pEqnIncomp.flux();

                    surfaceScalarField mSfGradp
                    (
                        "mSfGradp",
                        pEqnIncomp.flux()/rAUf
                    );

                    forAll(fluid.movingPhases(), movingPhasei)
                    {
                        phaseModel& phase = fluid_.movingPhases()[movingPhasei];

                        phase.phiRef() =
                            phiHbyAs[phase.index()]
                          + alpharAUfs[phase.index()]*mSfGradp;

                        // Set the phase dilatation rate
                        phase.divU(-pEqnComps[phase.index()] & p_rgh);
                    }

                    // Optionally relax pressure for velocity correction
                    p_rgh.relax();

                    mSfGradp = pEqnIncomp.flux()/rAUf;

                    // Momentum corrector
                    forAll(fluid.movingPhases(), movingPhasei)
                    {
                        phaseModel& phase = fluid_.movingPhases()[movingPhasei];
                        volVectorField& UAlpha = phase.URef();

                        if (faceMomentum)
                        {
                            UAlpha = fvc::reconstruct(phase.phi());
                        }
                        else
                        {
                            UAlpha =
                                1.0
                               /(
                                    HUEqns[phase.index()].A()
                                   /max(relaxU[phase.index()], small)
                                  + ddtUEqns[phase.index()].A()
                                  + dmdtUEqns[phase.index()].A()
                                  + fvModelsUEqns[phase.index()].A()
                                  + dragDiag[phase.index()]
                                  + vmDiag[phase.index()]
                                )
                               *(
                                    HUEqns[phase.index()].A()
                                   *HbyAs[phase.index()]
                                  + (1 - relaxU[phase.index()])
                                   /max(relaxU[phase.index()], small)
                                   *HUEqns[phase.index()].A()*UAlpha.prevIter()
                                  + ddtUEqns[phase.index()].H()
                                  + dmdtUEqns[phase.index()].H()
                                  + fvModelsUEqns[phase.index()].H()
                                  + dragSource[phase.index()]
                                  + vmSource[phase.index()]
                                  + buoyancySource[phase.index()]
                                )
                              + fvc::reconstruct
                                (
                                    phigs[phase.index()]
                                  + alpharAUfs[phase.index()]*mSfGradp
                                  + rHUfs[phase.index()]*rAUfs[phase.index()]
                                   *forcesSourcef[phase.index()]
                                );
                        }

                        UAlpha.correctBoundaryConditions();
                        fvConstraints().constrain(UAlpha);

                        MRF.makeRelative(phase.phiRef());
                    }
                }
            }

            // Update the mixture properties and static pressure
            U_ = fluid.U();
            p_ = p_rgh + rho*buoyancy.gh;
            fvConstraints().constrain(p_);

            // Account for static pressure reference
            if (p_rgh.needReference() && fluid.incompressible())
            {
                p_ += dimensionedScalar
                (
                    "p",
                    p.dimensions(),
                    pressureReference.refValue()
                  - getRefCellValue(p, pressureReference.refCell())
                );
            }

            // Limit p_rgh
            p_rgh = p - rho*buoyancy.gh;

            // Update densities from change in p_rgh
            forAll(phases, phasei)
            {
                phaseModel& phase = phases_[phasei];
                phase.thermoRef().rho() +=
                    phase.thermo().psi()*(p_rgh - p_rgh_0);
            }

            // Update mass transfer rates for change in p_rgh
            forAll(phases, phasei)
            {
                if (dmdts.set(phasei) && d2mdtdps.set(phasei))
                {
                    dmdts[phasei] += d2mdtdps[phasei]*(p_rgh - p_rgh_0);
                }
            }

            // Correct p_rgh for consistency with p and the updated densities
            rho_ = fluid.rho();
            p_rgh = p - rho*buoyancy.gh;
            p_rgh.correctBoundaryConditions();
        }

        alphafs.clear();

        HUEqns.clear();
        relaxU.clear();
        ddtUEqns.clear();
        ddtPhiCoeffs.clear();
        dmdtUEqns.clear();
        dmdtPhiCoeffs.clear();
        fvModelsUEqns.clear();

        rHUs.clear();
        rHUfs.clear();
        rAs.clear();
        rAfs.clear();

        dragDiag.clear();
        dragDiagf.clear();
        dragPCoefff.clear();
        vmDiag.clear();
        vmDiagf.clear();

        rAUfs.clear();

        if (!fluid.implicitPhasePressure())
        {
            rHAUfs.clear();
        }

        fluid_.correctKinematics();
    }
    else
    {
        PtrList<fvScalarMatrix> pEqnComps
        (
            compressibilityEqns(fluid.dmdts(), fluid.d2mdtdps())
        );

        forAll(phases, phasei)
        {
            phases_[phasei].divU(-pEqnComps[phasei] & p_rgh);
        }
    }
}


// ************************************************************************* //
