/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "cipsaMultiphaseEuler.H"
#include "fvmDiv.H"
#include "fvmSup.H"
#include "fvcDiv.H"

// * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * * //

void Foam::solvers::cipsaMultiphaseEuler::updateVmDiag()
{
    fluid.fillFields("vmDiag", dimensionSet(1, -3, -1, 0, 0), vmDiag);
    fluid.fillFields("vmDiagf", dimensionSet(1, -3, -1, 0, 0), vmDiagf);

    forAllConstIter(phaseSystem::VmTable, fluid.Vms(), VmIter)
    {
        const volScalarField& Vm(*VmIter());
        const phaseInterface interface(fluid, VmIter.key());

        forAllConstIter(phaseInterface, interface, interfaceIter)
        {
            const phaseModel& phase = interfaceIter();

            vmDiag[phase.index()] += byDt(Vm);
            vmDiagf[phase.index()] += byDt(fvc::interpolate(Vm));
        }
    }
}


Foam::PtrList<Foam::volVectorField>
Foam::solvers::cipsaMultiphaseEuler::calcVmSource()
{
    PtrList<volVectorField> vmSource(phases.size());
    fluid.fillFields("vmSource", dimensionSet(1, -2, -2, 0, 0), vmSource);

    forAllConstIter(phaseSystem::VmTable, fluid.Vms(), VmIter)
    {
        const volScalarField& Vm(*VmIter());
        const phaseInterface interface(fluid, VmIter.key());

        forAllConstIter(phaseInterface, interface, interfaceIter)
        {
            const phaseModel& phase = interfaceIter();
            const phaseModel& otherPhase = interfaceIter.otherPhase();

            vmSource[phase.index()] +=
                Vm
               *byDt
                (
                    phase.U()().oldTime()
                  + otherPhase.U()
                  - otherPhase.U()().oldTime()
                );
        }
    }

    return vmSource;
}


Foam::PtrList<Foam::surfaceScalarField>
Foam::solvers::cipsaMultiphaseEuler::calcVmSourcef()
{
    PtrList<surfaceScalarField> vmSourcef(phases.size());
    fluid.fillFields("vmSourcef", dimensionSet(0, 3, -1, 0, 0), vmSourcef);

    forAllConstIter(phaseSystem::VmTable, fluid.Vms(), VmIter)
    {
        const volScalarField& Vm(*VmIter());
        const phaseInterface interface(fluid, VmIter.key());

        forAllConstIter(phaseInterface, interface, interfaceIter)
        {
            const phaseModel& phase = interfaceIter();
            const phaseModel& otherPhase = interfaceIter.otherPhase();

            vmSourcef[phase.index()] +=
                rHUfs[phase.index()]
               *fvc::interpolate(Vm)
               *byDt
                (
                    MRF.absolute(phase.phi()().oldTime())
                  + MRF.absolute(otherPhase.phi())
                  - MRF.absolute(otherPhase.phi()().oldTime())
                );
        }
    }

    return vmSourcef;
}


void Foam::solvers::cipsaMultiphaseEuler::updateVmConvectionEqn()
{
    if (fluid.Vms().empty())
    {
        return;
    }

    // Create U & grad(U) fields
    PtrList<fvVectorMatrix> UgradUs(phases.size());
    forAll(fluid.movingPhases(), movingPhasei)
    {
        const phaseModel& phase = fluid.movingPhases()[movingPhasei];
        const volVectorField& UAlpha = phase.U();
        const surfaceScalarField& phiAlpha = phase.phi();

        UgradUs.set
        (
            phase.index(),
            new fvVectorMatrix
            (
                fvm::div(phiAlpha, UAlpha)
              + fvm::SuSp(- fvc::div(phiAlpha), UAlpha)
            )
        );
    }

    // Add convective part of the virtual mass force
    forAllConstIter(phaseSystem::VmTable, fluid.Vms(), VmIter)
    {
        const volScalarField& Vm(*VmIter());
        const phaseInterface interface(fluid, VmIter.key());

        forAllConstIter(phaseInterface, interface, interfaceIter)
        {
            const phaseModel& phase = interfaceIter();
            const phaseModel& otherPhase = interfaceIter.otherPhase();

            HUEqns[phase.index()] +=
                Vm
               *(
                    UgradUs[phase.index()]
                  - (UgradUs[otherPhase.index()] & otherPhase.U())
                );
        }
    }
}


// ************************************************************************* //
