/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::functionObjects::wallBoilingProperties

Description
    This function looks up wall boiling wall functions and collects and writes
    out out the following data:

      - Bubble departure diameter
      - Bubble departure frequency
      - Nucleation site density
      - Effective liquid fraction at the wall
      - Quenching heat flux
      - Evaporative heat flux

    Example of function object specification:
    \verbatim
    writeWallBoilingProperties
    {
        type            wallBoilingProperties;
        functionObjectLibs ( "libmultiphaseEulerFoamFunctionObjects.so" );
        writeControl    writeTime;
        phase           liquid;
    }
    \endverbatim

Usage
    \table
        Property | Description                | Required   | Default value
        type     | type name: wallBoilingProperties     | yes        |
        phase    | phase name                 | yes         | none
    \endtable

SourceFiles
    wallBoilingProperties.C

\*---------------------------------------------------------------------------*/

#ifndef wallBoilingProperties_H
#define wallBoilingProperties_H

#include "fvMeshFunctionObject.H"
#include "phaseSystem.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace functionObjects
{

/*---------------------------------------------------------------------------*\
                         Class wallBoilingProperties Declaration
\*---------------------------------------------------------------------------*/

class wallBoilingProperties
:
    public fvMeshFunctionObject
{
    // Private data

        //- Phase model
        const phaseModel& phase_;

        //- Constant access to phaseSystem
        const phaseSystem& fluid_;


public:

    //- Runtime type information
    TypeName("wallBoilingProperties");


    // Constructors

        //- Construct from Time and dictionary
        wallBoilingProperties
        (
            const word& name,
            const Time& runTime,
            const dictionary& dict
        );

        //- Disallow default bitwise copy construction
        wallBoilingProperties(const wallBoilingProperties&) = delete;


    //- Destructor
    virtual ~wallBoilingProperties();


    // Member Functions

        //- Read the wallBoilingProperties data
        virtual bool read(const dictionary&);

        //- Return the list of fields required
        virtual wordList fields() const
        {
            return wordList::null();
        }

        //- Calculate the wallBoilingProperties field
        virtual bool execute();

        //- Write the wallBoilingProperties field
        virtual bool write();


    // Member Operators

        //- Disallow default bitwise assignment
        void operator=(const wallBoilingProperties&) = delete;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace functionObjects
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
