/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "degassingPhases.H"
#include "fvMatrix.H"
#include "fvmSup.H"
#include "surfaceInterpolate.H"
#include "addToRunTimeSelectionTable.H"
#include "nonConformalCyclicFvPatchFields.H"
#include "sizeGroup.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace fv
{
    defineTypeNameAndDebug(degassingPhases, 0);
    addToRunTimeSelectionTable
    (
        fvModel,
        degassingPhases,
        dictionary
    );
}
}


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

bool Foam::fv::degassingPhases::isDegassingPhase(const word& fieldName) const
{
    return pos0(findIndex(degassingPhases_, IOobject::group(fieldName)));
}


Foam::scalar
Foam::fv::degassingPhases::patchCellVolume(const wordList& patches) const
{
    scalar V(0.0);

    forAll(patches, patchi)
    {
        const fvPatch& patchField = mesh().boundary()[patches[patchi]];

        forAll(patchField.faceCells(), i)
        {
            V += mesh().V()[patchField.faceCells()[i]];
        }
    }

    reduce(V, sumOp<scalar>());

    return V;
}


template<class Type>
void Foam::fv::degassingPhases::addSupType
(
    const volScalarField& alpha,
    fvMatrix<Type>& eqn,
    const word& fieldName
) const
{
    const VolField<Type>& psi = eqn.psi();
    const scalar rDeltaT = 1.0/mesh().time().deltaT().value();

    volScalarField::Internal Sp
    (
        IOobject
        (
            name() + ":" + fieldName + ":Sp",
            mesh().time().name(),
            mesh()
        ),
        mesh(),
        dimensionedScalar(dimless/dimTime, Zero),
        false
    );

    if (isDegassingPhase(fieldName))
    {
        forAll(cells_, i)
        {
            Sp[cells_[i]] -= alpha[cells_[i]]*rDeltaT;
        }
    }
    else
    {
        scalar volumeRate = 0.0;
        scalar boundaryRate = 0.0;

        forAll(degassingPhases_, namei)
        {
            const surfaceScalarField& phi =
                mesh().lookupObject<surfaceScalarField>
                (
                    IOobject::groupName("phi", degassingPhases_[namei])
                );

            const volScalarField& rhoAlpha =
                mesh().lookupObject<volScalarField>
                (
                    IOobject::groupName("rho", degassingPhases_[namei])
                );

            const volScalarField& degassingAlpha =
                mesh().lookupObject<volScalarField>
                (
                    IOobject::groupName(alphaName_, degassingPhases_[namei])
                );

            forAll(mesh().boundary(), patchi)
            {
                if
                (
                    mesh().boundary()[patchi].coupled()
                 || isA<nonConformalCyclicFvPatch>(mesh().boundary()[patchi])
                )
                {
                    continue;
                }

                boundaryRate +=
                    sum
                    (
                       phi.boundaryField()[patchi]
                      *degassingAlpha.boundaryField()[patchi]
                    );
            }

            const fvModels& fvModels(fvModels::New(mesh()));
            const PtrListDictionary<fvModel>& modelList = (fvModels);

            forAll(modelList, i)
            {
                const fvModel& model = modelList[i];

                if
                (
                    model.name() != this->name()
                 && model.addsSupToField(rhoAlpha.name())
                )
                {
                    const volScalarField source
                    (
                        model.source(degassingAlpha, rhoAlpha) & rhoAlpha
                    );

                    volumeRate -= sum(source*mesh().cellVolumes()/rhoAlpha);
                }
            }
        }

        reduce(boundaryRate, sumOp<scalar>());
        reduce(volumeRate, sumOp<scalar>());

        scalar rate = (volumeRate + boundaryRate)/V_;

        if (debug)
        {
            Info<< name() <<": " << fieldName << ": rate: " << rate << endl;
        }

        forAll(cells_, i)
        {
            Sp[cells_[i]] = rate;

            forAll(degassingPhases_, namei)
            {
                const volScalarField& degassingAlpha =
                    mesh().lookupObject<volScalarField>
                    (
                        IOobject::groupName(alphaName_, degassingPhases_[namei])
                    );

                Sp[cells_[i]] += degassingAlpha[cells_[i]]*rDeltaT;
            }
        }
    }

    if (debug)
    {
        Info<< name() <<": " << fieldName << ": Sp min, max: "
            << min(Sp).value() << ", " << max(Sp).value() << endl;

        if (mesh().time().writeTime()) Sp.write();
    }

    eqn += fvm::SuSp(Sp, psi);
}


template<class Type>
void Foam::fv::degassingPhases::addSupType
(
    const volScalarField& alpha,
    const volScalarField& rho,
    fvMatrix<Type>& eqn,
    const word& fieldName
) const
{
    const scalar rDeltaT = 1.0/mesh().time().deltaT().value();

    volScalarField::Internal Sp
    (
        IOobject
        (
            name() + ":" + fieldName + ":Sp",
            mesh().time().name(),
            mesh()
        ),
        mesh(),
        dimensionedScalar(dimDensity/dimTime, Zero),
        false
    );

    forAll(cells_, i)
    {
        Sp[cells_[i]] -= rho[cells_[i]]*alpha[cells_[i]]*rDeltaT;
    }

    eqn += fvm::SuSp(Sp, eqn.psi());
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::fv::degassingPhases::degassingPhases
(
    const word& name,
    const word& modelType,
    const fvMesh& mesh,
    const dictionary& dict
)
:
    fvModel(name, modelType, mesh, dict),
    degassingPhases_(dict.lookup<wordList>("degassingPhases")),
    continuousPhase_(dict.lookupOrDefault<word>("continuousPhase", "default")),
    alphaName_(dict.lookupOrDefault<word>("alphaName", "alpha")),
    fluid_(mesh.lookupObject<phaseSystem>(phaseSystem::propertiesName)),
    patchNames_(dict.lookup<wordList>("patchNames"))
{
    read(dict);

    if (continuousPhase_ == "default")
    {
        if (fluid_.continuousPhases().size() == 1)
        {
            fieldNames_.append(fluid_.continuousPhases()[0].rho()().name());
            continuousPhase_ = fluid_.continuousPhases()[0].name();
        }
        else
        {
            FatalErrorInFunction
                << "Exactly one continuous phase is required, but there are "
                << fluid_.continuousPhases().size() << " specified."
                << endl << exit(FatalError);
        }
    }
    else
    {
        const phaseModel& phase = fluid_.phases()[continuousPhase_];
        fieldNames_.append(phase.rho()().name());
    }

    // List of solution variables to apply correction sink term to
    const wordList solutionVariables
    (
        wordList{"U", "k", "R", "epsilon", "omega"}
    );

    forAll(degassingPhases_, phasei)
    {
        const phaseModel& phase = fluid_.phases()[degassingPhases_[phasei]];

        // Add phase density field
        fieldNames_.append(phase.rho()().name());

        // Add fields of solution variables
        forAll(solutionVariables, variablei)
        {
            const word variable = solutionVariables[variablei];
            const word fieldName =
                IOobject::groupName(variable, phase.name());

            if (pos0(findIndex(mesh.names(), fieldName)))
            {
                fieldNames_.append(fieldName);
            }
        }

        // Add field of solution variable for energy
        if (!phase.isothermal())
        {
            fieldNames_.append(phase.thermo().he().name());
        }

        // Add size group fields
        if (isA<diameterModels::velocityGroup>(phase.dPtr()()))
        {
            const diameterModels::velocityGroup& velGroup=
                refCast<const diameterModels::velocityGroup>
                (
                    phase.dPtr()()
                );

            forAll(velGroup.sizeGroups(), i)
            {
                const diameterModels::sizeGroup* fi =
                    velGroup.sizeGroups()(i);

                fieldNames_.append(fi->name());
            }
        }
    }

    if (debug)
    {
        word phases = degassingPhases_[0];
        for (Foam::label i=1; i<degassingPhases_.size(); i++)
        {
            phases += ", " + degassingPhases_[i];
        }
        Info<< "    Degassing Phases: " << phases << nl
            << "    Continuous phase: " << continuousPhase_ << endl;

    }

    forAll(patchNames_, patchi)
    {
        const fvPatch& patchField = mesh.boundary()[patchNames_[patchi]];

        forAll(patchField.faceCells(), i)
        {
            cells_.append(patchField.faceCells()[i]);
        }
    }

    V_ = patchCellVolume(patchNames_);
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::fv::degassingPhases::~degassingPhases()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

bool Foam::fv::degassingPhases::addsSupToField(const word& fieldName) const
{
    return pos0(findIndex(fieldNames_, fieldName));
}


Foam::wordList Foam::fv::degassingPhases::addSupFields() const
{
    return fieldNames_;
}


FOR_ALL_FIELD_TYPES
(
    IMPLEMENT_FV_MODEL_ADD_RHO_SUP,
    fv::degassingPhases
);


FOR_ALL_FIELD_TYPES
(
    IMPLEMENT_FV_MODEL_ADD_ALPHA_RHO_SUP,
    fv::degassingPhases
);


void Foam::fv::degassingPhases::topoChange(const polyTopoChangeMap&)
{}


void Foam::fv::degassingPhases::mapMesh(const polyMeshMap& map)
{}


void Foam::fv::degassingPhases::distribute(const polyDistributionMap&)
{}


bool Foam::fv::degassingPhases::movePoints()
{
    V_ = patchCellVolume(patchNames_);

    return true;
}


// ************************************************************************* //
