/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::subGridSurfaceTensionModel::WeberNumberModel

Description
    Abstract base class for sub-grid surface tension models with effective
    Weber number approach.

    Reference:
    \verbatim
        Ketterl, S., Reißmann, M., Klein, M. (2019).
        Large eddy simulation of multiphase flows using the volume of fluid
        method: Part 2 - A-posteriori analysis of liquid jet atomization.
        Experimental and Computational Multiphase Flow, 1(3), 201-211.
    \endverbatim

SourceFiles
    WeberNumberModel.C

\*---------------------------------------------------------------------------*/

#ifndef WeberNumberModel_H
#define WeberNumberModel_H

#include "subGridSurfaceTensionModel.H"
#include "SGSdelta.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

namespace subGridSurfaceTensionModels
{

/*---------------------------------------------------------------------------*\
                      Class WeberNumberModel Declaration
\*---------------------------------------------------------------------------*/

class WeberNumberModel
:
    public subGridSurfaceTensionModel
{
    // Private Data

        //- Run-time selectable delta model
        autoPtr<Foam::SGSdelta> delta_;


    // Private Member Functions

        //- Effective surface tension coefficient
        tmp<volScalarField> sigmaEff() const;

        //- Sub-grid surface tension force
        virtual tmp<volVectorField> subGridSurfaceTensionForce() const;

        //- Critical Weber number
        virtual tmp<volScalarField> WeCrit() const = 0;


protected:

    // Protected Member Functions

        // Access

            //- Access function to filter width
            inline const volScalarField& delta() const
            {
                return delta_();
            }


        //- Turbulent surface tension coefficient
        virtual tmp<volScalarField> sigmat() const;

        //- Face based turbulent surface tension coefficient
        virtual tmp<surfaceScalarField> sigmatf() const;

        //- Velocity of interface
        tmp<volVectorField> U() const;

        //- Write out effective surface tension coefficient and SGS-ST force
        virtual bool write() const;


public:

    // Constructors

        //- Construct from a dictionary and an interface
        WeberNumberModel
        (
            const dictionary& dict,
            const phaseInterface& interface
        );


    //- Destructor
    virtual ~WeberNumberModel();


    // Member Functions

        //- Sub grid surface tension flux
        virtual tmp<surfaceScalarField> subGridSurfaceTensionForcef() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace subGridSurfaceTensionModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
