/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::subGridSurfaceTensionModel

Description
    Base class for modeling the sub-grid surface tension flux.

SourceFiles
    subGridSurfaceTensionModel.C
    subGridSurfaceTensionModelNew.C

\*---------------------------------------------------------------------------*/

#ifndef subGridSurfaceTensionModel_H
#define subGridSurfaceTensionModel_H

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "volFields.H"
#include "dictionary.H"
#include "fvc.H"
#include "segregatedPhaseInterface.H"
#include "runTimeSelectionTables.H"

namespace Foam
{

class phaseModel;

/*---------------------------------------------------------------------------*\
                 Class subGridSurfaceTensionModel Declaration
\*---------------------------------------------------------------------------*/

class subGridSurfaceTensionModel
{
    // Private Data

        //- Flag to print the model coeffs at run-time
        Switch printCoeffs_;


    // Private Member Functions

        //- Sub grid surface tension force
        virtual tmp<volVectorField> subGridSurfaceTensionForce() const = 0;


protected:

    // Protected Data

        //- Interface
        const segregatedPhaseInterface interface_;

        //- Model coefficients dictionary
        dictionary coeffDict_;


    // Protected Member Functions

        //- Print model coefficients
        void printCoeffs(const word& type);

        //- Write out effective sub-grid surface tension force
        virtual bool write() const;


public:

    //- Runtime type information
    TypeName("subGridSurfaceTensionModel");


    // Declare runtime construction

        declareRunTimeSelectionTable
        (
            autoPtr,
            subGridSurfaceTensionModel,
            dictionary,
            (
                const dictionary& dict,
                const phaseInterface& interface
            ),
            (dict, interface)
        );


    // Static Data Members

        //- Does this model require correcting on fixed flux boundaries?
        static const bool correctFixedFluxBCs = true;


    // Constructors

        //- Construct from a dictionary and an interface
        subGridSurfaceTensionModel
        (
            const dictionary& dict,
            const phaseInterface& interface
        );


    //- Destructor
    virtual ~subGridSurfaceTensionModel();


    // Selectors

        static autoPtr<subGridSurfaceTensionModel> New
        (
            const dictionary& dict,
            const phaseInterface& interface,
            const bool outer=true
        );


    // Member Functions

        //- Sub grid surface tension flux
        virtual tmp<surfaceScalarField> subGridSurfaceTensionForcef() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
