/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::aspectRatioModels::ZiegenheinLucas

Description
    Aspect ratio model of Ziegenhein and Lucas which uses the functional form of
    Wellek et al. but with a different coefficient A and exponent B.

    \f[
        d_{\perp} = d_b  \sqrt[3]{1 + A \mathrm{Eo}^{B}}
    \f]

    where

    \vartable
        d_{\perp}   | Bubble diameter perpendicular to main motion
        d_b         | Bulk bubble diameter
        \mathrm{Eo} | Eötvös number
    \endvartable

    Both values depend on the degree of contamination and the involved
    substances and are therefore user-modifiable. The default values set here
    are not explicitly stated by Ziegenhein et. al, but can be extracted from
    the experimental data.

    References:
    \verbatim
        Ziegenhein, T., & Lucas, D. (2017).
        Observations on bubble shapes in bubble columns under different flow
        conditions.
        Experimental Thermal and Fluid Science, 85, 248-256.
    \endverbatim

    \verbatim
        Wellek, R. M., Agrawal, A. K., & Skelland, A. H. P. (1966).
        Shape of liquid drops moving in liquid media.
        AIChE Journal, 12(5), 854-862.
    \endverbatim

Usage
    \table
        Property     | Description             | Required    | Default value
        A            | Coefficient A           | no          | 0.65
        B            | Coefficient B           | no          | 0.35
    \endtable

SourceFiles
    ZiegenheinLucas.C

\*---------------------------------------------------------------------------*/

#ifndef ZiegenheinLucas_H
#define ZiegenheinLucas_H

#include "aspectRatioModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace aspectRatioModels
{

/*---------------------------------------------------------------------------*\
                       Class ZiegenheinLucas Declaration
\*---------------------------------------------------------------------------*/

class ZiegenheinLucas
:
    public aspectRatioModel
{
    // Private Data

        //- Optional coefficient A, defaults to 0.65
        const scalar A_;

        //- Optional exponent B, defaults to 0.35
        const scalar B_;


public:

    //- Runtime type information
    TypeName("ZiegenheinLucas");


    // Constructors

        //- Construct from a dictionary and an ordered phase pair
        ZiegenheinLucas
        (
            const dictionary& dict,
            const phaseInterface& interface
        );


    //- Destructor
    virtual ~ZiegenheinLucas();


    // Member Functions

        //- Aspect ratio
        virtual tmp<volScalarField> E() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace aspectRatioModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
