/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::dragModels::ShiRzehakDrag

Description
    Shi and Rzehak (2020) drag model for turbulent dispersed solid-liquid
    flows.

    \verbatim
        Shi, P., & Rzehak, R. (2020).
        Solid-liquid flow in stirred tanks: Euler-Euler / RANS modeling.
        Chemical Engineering Science, 227, 115875.
    \endverbatim

    Note: The particle relaxation time tauS uses the instantaneous relative
          velocity and SchillerNaumann drag coefficient.

SourceFiles
    ShiRzehakDrag.C

\*---------------------------------------------------------------------------*/

#ifndef ShiRzehakDrag_H
#define ShiRzehakDrag_H

#include "dispersedDragModel.H"
#include "phaseCompressibleMomentumTransportModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace dragModels
{

class SchillerNaumann;

/*---------------------------------------------------------------------------*\
                        Class ShiRzehakDrag Declaration
\*---------------------------------------------------------------------------*/

class ShiRzehakDrag
:
    public dispersedDragModel
{
    // Private Data

        //- SchillerNaumann drag model
        autoPtr<SchillerNaumann> SchillerNaumann_;

        //- Constant to describe the Eulerian longitudinal
        //  integral length scale
        const dimensionedScalar CLambda_;

        //- Constant to describe the Lagrangian integral timescale
        //  following the fluid motion
        const dimensionedScalar CT_;


public:

    //- Runtime type information
    TypeName("ShiRzehak");


    // Constructors

        //- Construct from a dictionary and an interface
        ShiRzehakDrag
        (
            const dictionary& dict,
            const phaseInterface& interface,
            const bool registerObject
        );


    //- Destructor
    virtual ~ShiRzehakDrag();


    // Member Functions

        //- Drag coefficient
        virtual tmp<volScalarField> CdRe() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace dragModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
