/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::dragModels::constantDragCoefficient

Description
    Numerical drag model for bubbly flows. The drag law is formulated with a
    constant coefficient Cd, which has to be defined by the user.

Interfacial momentum transfer due to drag
\f[
    \vec{M}_d = -\vec{M}_c = K\,\left(\vec{U}_c-\vec{U}_d\right)
\f]

Drag function K
\f[
    K = \alpha_d Ki
\f]

Phase specific drag function Ki
\f[
    Ki = 0.75 CdRe\rho_c\nu_c/d^2
\f]

Combined drag coefficient and Reynolds number CdRe
\f[
    CdRe = c_d max(Re, Re_{res})
\f]

Usage
    \table
        Parameter      | Name       | Description        | Default value
        \f$c_d\f$      | Cd         | drag coefficient   | -
        \f$Re_{res}\f$ | residualRe | minimal applied Re | -
    \endtable

SourceFiles
    constantDragCoefficient.C

\*---------------------------------------------------------------------------*/

#ifndef constantDragCoefficient_H
#define constantDragCoefficient_H

#include "dispersedDragModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace dragModels
{

/*---------------------------------------------------------------------------*\
                   Class constantDragCoefficient Declaration
\*---------------------------------------------------------------------------*/

class constantDragCoefficient
:
    public dispersedDragModel
{
    // Private Data

        //- Drag coefficient
        const dimensionedScalar Cd_;

        //- Residual Reynolds Number
        const dimensionedScalar residualRe_;


public:

    //- Runtime type information
    TypeName("constantDragCoefficient");


    // Constructors

        //- Construct from a dictionary and an interface
        constantDragCoefficient
        (
            const dictionary& dict,
            const phaseInterface& interface,
            const bool registerObject
        );


    //- Destructor
    virtual ~constantDragCoefficient();


    // Member Functions

        //- Drag coefficient
        virtual tmp<volScalarField> CdRe() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace dragModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
