/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::dragModels::resolutionAdaptiveInterfaceDrag

Description
    A wrapper class around two underlying drag models, which is intended for
    simulations involving large interfaces. The latter are adaptively described
    in either a resolving (R) or in an under-resolving manner (UR) or as a
    mixture of both. Both basic drag model formulations need to be selected
    separately. The degree of under-resolution is controlled via the under-
    resolution indicator function \f$f_\mathrm{UR}\f$.

    The resulting drag coefficient is calculated as follows (similarly to a
    parallel electric circuit with two resistors):
    \f[
        K = \frac{K_\mathrm{UR} K_\mathrm{R}}
                 {K_\mathrm{UR} + f_\mathrm{UR} K_\mathrm{R}}.
    \f]

SourceFiles
    resolutionAdaptiveInterfaceDrag.C

\*---------------------------------------------------------------------------*/

#ifndef resolutionAdaptiveInterfaceDrag_H
#define resolutionAdaptiveInterfaceDrag_H

#include "dragModel.H"
#include "segregatedPhaseInterface.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace dragModels
{

/*---------------------------------------------------------------------------*\
               Class resolutionAdaptiveInterfaceDrag Declaration
\*---------------------------------------------------------------------------*/

class resolutionAdaptiveInterfaceDrag
:
    public dragModel
{
    // Private Data

        //- Interface
        const segregatedPhaseInterface interface_;

        //- Pointer to the underlying resolving drag model
        autoPtr<dragModel> resolvingDragModel_;

        //- Pointer to the underlying under-resolving drag model
        autoPtr<dragModel> underResolvingDragModel_;


public:

    //- Runtime type information
    TypeName("resolutionAdaptiveInterface");


    // Constructors

        //- Construct from a dictionary and an interface
        resolutionAdaptiveInterfaceDrag
        (
            const dictionary& dict,
            const phaseInterface& interface,
            const bool registerObject
        );


    //- Destructor
    virtual ~resolutionAdaptiveInterfaceDrag();


    // Member Functions

        //- Return the drag coefficient K
        //  used in the momentum equations
        //    ddt(alpha1*rho1*U1) + ... = ... K*(U1-U2)
        //    ddt(alpha2*rho2*U2) + ... = ... K*(U2-U1)
        virtual tmp<volScalarField> K() const;

        //- Return the drag coefficient Kf
        //  used in the face-momentum equations
        virtual tmp<surfaceScalarField> Kf() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace dragModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
