/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "LiaoLucas.H"
#include "heatTransferPhaseSystem.H"
#include "interfaceSaturationTemperatureModel.H"
#include "phaseCompressibleMomentumTransportModel.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace heatTransferModels
{
    defineTypeNameAndDebug(LiaoLucas, 0);
    addToRunTimeSelectionTable(heatTransferModel, LiaoLucas, dictionary);
}
}

using Foam::constant::mathematical::pi;


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::heatTransferModels::LiaoLucas::LiaoLucas
(
    const dictionary& dict,
    const phaseInterface& interface,
    const bool registerObject
)
:
    heatTransferModel(dict, interface, registerObject),
    interface_
    (
        interface.modelCast<heatTransferModel, dispersedPhaseInterface>()
    )
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::heatTransferModels::LiaoLucas::~LiaoLucas()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::heatTransferModels::LiaoLucas::K(const scalar residualAlpha) const
{
    const phaseModel& liquid = interface_.continuous();
    const phaseModel& vapor = interface_.dispersed();
    const rhoThermo& lThermo = liquid.thermo();
    const rhoThermo& vThermo = vapor.thermo();
    const phaseSystem& fluid = interface_.continuous().fluid();
    const word volatileSpecie(fluid.lookupOrDefault<word>("volatile", "none"));

    const phaseCompressible::momentumTransportModel& lTurbulence =
        interface_.mesh().lookupObject
        <
            phaseCompressible::momentumTransportModel
        >
        (
            IOobject::groupName
            (
                momentumTransportModel::typeName,
                liquid.name()
            )
        );

    if
    (
       !fluid.foundInterfacialModel<interfaceSaturationTemperatureModel>
        (
            interface_
        )
    )
    {
        FatalErrorInFunction
            << "Interface saturation temperature model for phase interface "
            << interface_.name() << " required for " << this->type()
            << " heat transfer model." << exit(FatalError);
    }

    const interfaceSaturationTemperatureModel& satModel =
        fluid.lookupInterfacialModel<interfaceSaturationTemperatureModel>
        (
            interface_
        );

    const volScalarField L
    (
        volatileSpecie != "none"
      ? refCast<const heatTransferPhaseSystem>(fluid)
        .Li
         (
            interface_,
            volatileSpecie,
            fluid.dmdtf(interface_),
            satModel.Tsat(lThermo.p()),
            heatTransferPhaseSystem::latentHeatScheme::upwind
         )
      : refCast<const heatTransferPhaseSystem>(fluid)
        .L
         (
            interface_,
            fluid.dmdtf(interface_),
            satModel.Tsat(lThermo.p()),
            heatTransferPhaseSystem::latentHeatScheme::upwind
         )
    );

    const volScalarField Ja
    (
        lThermo.Cp()*lThermo.rho()*(lThermo.T() - satModel.Tsat(lThermo.p()))
       /(vThermo.rho()*L)
    );

    const volScalarField alphahe(lThermo.kappa()/lThermo.Cpv());

    const volScalarField Pe
    (
        interface_.magUr()*vapor.d()*lThermo.rho()/alphahe
    );

    const volScalarField tauTurb(sqrt(lThermo.nu()/lTurbulence.epsilon()));

    const volScalarField Nu
    (
        12/pi*Ja
      + 2*sqrt(Pe/pi)
      + 2*vapor.d()/sqrt(pi*alphahe/lThermo.rho()*tauTurb)
    );

    return
        6
       *max(interface_.dispersed(), residualAlpha)
       *interface_.continuous().thermo().kappa()
       *Nu
       /sqr(interface_.dispersed().d());
}


// ************************************************************************* //
