/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "BhutaniBritoParada.H"
#include "phaseSystem.H"
#include "addToRunTimeSelectionTable.H"
#include "rhoMulticomponentThermo.H"
#include "phaseCompressibleMomentumTransportModel.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace phaseTransferModels
{
    defineTypeNameAndDebug(BhutaniBritoParada, 0);
    addToRunTimeSelectionTable
    (
        phaseTransferModel, BhutaniBritoParada, dictionary
    );
}
}

using namespace Foam::constant::mathematical;


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

inline Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::BhutaniBritoParada::dCrit
(
    const volScalarField& nu,
    const volScalarField& k,
    const volScalarField& eps,
    const volScalarField& rhoC,
    const volScalarField& rhoD
) const
{
    return
        sqrt
        (
            30*nu*rhoC*k
           /max
            (
                rhoD*eps,
                dimensionedScalar(dimensionSet(1, -1, -3, 0, 0), small)
            )
        );
}


inline Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::BhutaniBritoParada::uPrime
(
    const volScalarField& d,
    const volScalarField& nu,
    const volScalarField& eps,
    const volScalarField& rhoC,
    const volScalarField& rhoD
) const
{
    return
        0.4*pow(eps, 4.0/9)*pow(d, 7.0/9)/cbrt(nu)
       *pow(mag(rhoD - rhoC)/rhoC, 2.0/3);
}


inline Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::BhutaniBritoParada::nD
(
    const volScalarField& d,
    const volScalarField& alpha,
    const volScalarField& rhoA,
    const volScalarField& rhoD,
    const volScalarField& Y
) const
{
    return rhoA*alpha*Y*6.0/(rhoD*pi*pow(d, 3));
}


inline Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::BhutaniBritoParada::Re
(
    const volScalarField& d,
    const volScalarField& u,
    const volScalarField& nu
) const
{
    return d*u/nu;
}


inline Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::BhutaniBritoParada::Bond
(
    const volScalarField& dP,
    const volScalarField& dB,
    const volScalarField& rhoP,
    const volScalarField& rhoL,
    const volScalarField& eps,
    const uniformDimensionedVectorField& g,
    const volScalarField& sigma
) const
{
    return
        (
            sqr(dP)
           *(
                mag(rhoP - rhoL)*mag(g)
                + 1.9*rhoP*sqr(cbrt(eps))*cbrt(2.0/(dP + dB))
            )
          + 1.5*dP*(4*sigma/dB - dB*rhoL*mag(g))*sqr(sin(pi - 0.5*theta_))
        )
       /mag(6.0*sigma*sin(pi - 0.5*theta_)*sin(pi + 0.5*theta_));
}


inline Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::BhutaniBritoParada::t
(
    const volScalarField& d
) const
{
    return
        dimensionedScalar
        (
            dimensionSet(0, -B_, 1, 0, 0), 75.0
        )
       /theta_*pow(d, B_);
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::phaseTransferModels::BhutaniBritoParada::BhutaniBritoParada
(
    const dictionary& dict,
    const phaseInterface& interface
)
:
    phaseTransferModel(dict, interface),
    interface_
    (
        interface.modelCast<phaseTransferModel, dispersedPhaseInterface>()
    ),
    detachedPhase_
    (
        interface.mesh().lookupObject<phaseModel>
        (
            IOobject::groupName("alpha", dict.lookup("detachedPhase"))
        )
    ),
    aggregatePhase_(interface.otherPhase(detachedPhase_)),
    continuousPhase_
    (
        aggregatePhase_.mesh().lookupObject<phaseModel>
        (
            IOobject::groupName("alpha", dict.lookup("continuousPhase"))
        )
    ),
    residualAlpha_
    (
        dimensionedScalar::lookupOrDefault
        (
            "residualAlpha",
            dict,
            10*aggregatePhase_.residualAlpha().value()
        )
    ),
    sign_(detachedPhase_.name() == interface_.dispersed().name() ? -1 : 1),
    species_(wordList(word(dict.lookup("species")))),
    carrierSpecies_(dict.lookup("carrierSpecies")),
    maxLoading_(dict.lookupOrDefault<scalar>("maxLoading", 1.0)),
    stability_(dict.lookupOrDefault<Switch>("stability", true)),
    detachment_(dict.lookupOrDefault<Switch>("detachment", true)),
    theta_(dict.lookupOrDefault<scalar>("theta", 75.0)),
    B_(dict.lookupOrDefault<scalar>("B", 0.6)),
    C_(dict.lookupOrDefault<scalar>("C", 2.0)),
    As_(dict.lookupOrDefault<scalar>("As", 1.0)),
    Psi_
    (
        IOobject
        (
            IOobject::groupName("Psi", aggregatePhase_.name()),
            aggregatePhase_.mesh().time().name(),
            aggregatePhase_.mesh(),
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        aggregatePhase_.mesh(),
        dimensionedScalar(dimDmdt, Zero)
    ),
    attachmentRatio_
    (
        IOobject
        (
            IOobject::groupName("attachmentRatio", aggregatePhase_.name()),
            aggregatePhase_.mesh().time().name(),
            aggregatePhase_.mesh(),
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        aggregatePhase_.mesh(),
        dimensionedScalar(dimless, Zero)
    ),
    surfaceLoading_
    (
        IOobject
        (
            IOobject::groupName("surfaceLoading", aggregatePhase_.name()),
            aggregatePhase_.mesh().time().name(),
            aggregatePhase_.mesh(),
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        aggregatePhase_.mesh(),
        dimensionedScalar(dimless, Zero)
    ),
    Z_
    (
        IOobject
        (
            IOobject::groupName("Z", aggregatePhase_.name()),
            aggregatePhase_.mesh().time().name(),
            aggregatePhase_.mesh(),
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        aggregatePhase_.mesh(),
        dimensionedScalar(dimVol/dimTime, Zero)
    ),
    ZPrime_
    (
        IOobject
        (
            IOobject::groupName("ZPrime", aggregatePhase_.name()),
            aggregatePhase_.mesh().time().name(),
            aggregatePhase_.mesh(),
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        aggregatePhase_.mesh(),
        dimensionedScalar(inv(dimTime), Zero)
    ),
    Pc_
    (
        IOobject
        (
            IOobject::groupName("Pc", aggregatePhase_.name()),
            aggregatePhase_.mesh().time().name(),
            aggregatePhase_.mesh(),
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        aggregatePhase_.mesh(),
        dimensionedScalar(dimless, Zero)
    ),
    Pa_
    (
        IOobject
        (
            IOobject::groupName("Pa", aggregatePhase_.name()),
            aggregatePhase_.mesh().time().name(),
            aggregatePhase_.mesh(),
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        aggregatePhase_.mesh(),
        dimensionedScalar(dimless, Zero)
    ),
    Ps_
    (
        IOobject
        (
            IOobject::groupName("Ps", aggregatePhase_.name()),
            aggregatePhase_.mesh().time().name(),
            aggregatePhase_.mesh(),
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        aggregatePhase_.mesh(),
        dimensionedScalar(dimless, Zero)
    )
{
    if(!interface.contains(detachedPhase_))
    {
        FatalErrorInFunction
            << "The detached phase is expected in the pair "
            << interface.name() << " for attachment modeling"
            << exit(FatalError);
    }
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::phaseTransferModels::BhutaniBritoParada::~BhutaniBritoParada()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

const Foam::hashedWordList&
Foam::phaseTransferModels::BhutaniBritoParada::species() const
{
    return species_;
}


Foam::HashPtrTable<Foam::volScalarField>
Foam::phaseTransferModels::BhutaniBritoParada::dmidtf() const
{
    HashPtrTable<volScalarField> result;

    const rhoMulticomponentThermo& thermo =
        aggregatePhase_.mesh().lookupObject<rhoMulticomponentThermo>
        (
            IOobject::groupName
            (
                physicalProperties::typeName,
                aggregatePhase_.name()
            )
        );

    const phaseCompressible::momentumTransportModel& turbulence =
        continuousPhase_.mesh().lookupObject
        <
            phaseCompressible::momentumTransportModel
        >
        (
            IOobject::groupName
            (
                momentumTransportModel::typeName,
                continuousPhase_.name()
            )
        );

    const phaseInterface interface(aggregatePhase_, continuousPhase_);

    const volScalarField& p(thermo.p());
    const volScalarField& T(thermo.T());

    const label& index =
        thermo.composition().index
        (
            aggregatePhase_.Y(carrierSpecies_)
        );

    const label& indexi =
        thermo.composition().index
        (
            aggregatePhase_.Y(species_[0])
        );

    const volScalarField& Yb = aggregatePhase_.Y(carrierSpecies_);
    const volScalarField& Yp = aggregatePhase_.Y(species_[0]);
    const volScalarField rhoP = thermo.composition().rho(indexi, p, T);
    const volScalarField rhoB = thermo.composition().rho(index, p, T);
    const volScalarField& rhoA = aggregatePhase_.rho();
    const volScalarField& rhoL = continuousPhase_.rho();
    const volScalarField dB = aggregatePhase_.d();
    const volScalarField dP = detachedPhase_.d();
    const volScalarField kL = turbulence.k();
    const volScalarField epsL = turbulence.epsilon();
    const volScalarField nuL = continuousPhase_.thermo().nu();

    const volScalarField dPCrit
    (
        IOobject::groupName("dCrit", detachedPhase_.name()),
        dCrit(nuL, kL, epsL, rhoL, rhoP)
    );

    const volScalarField dBCrit
    (
        IOobject::groupName("dCrit", aggregatePhase_.name()),
        dCrit(nuL, kL, epsL, rhoL, 0.5*rhoL)
    );

    const volScalarField uPrimeP
    (
        IOobject::groupName("UPrime", detachedPhase_.name()),
        uPrime(dP, nuL, epsL, rhoL, rhoP)
    );

    const volScalarField uPrimeB
    (
        IOobject::groupName("UPrime", aggregatePhase_.name()),
        uPrime(dB, nuL, epsL, rhoL, rhoB)
    );

    attachmentRatio_ = pow3(dB/dP)*Yp*rhoB/(max(Yb, small)*rhoP);
    surfaceLoading_ = 0.25*attachmentRatio_*sqr(dP/dB);

    const volScalarField nBa
    (
        IOobject::groupName("availableBubbles", aggregatePhase_.name()),
        (1 - surfaceLoading_/maxLoading_)
       *nD(dB, aggregatePhase_, rhoA, rhoB, Yb)
    );

    const volScalarField nBl
    (
        IOobject::groupName("loadedBubbles", aggregatePhase_.name()),
        surfaceLoading_/maxLoading_*nD(dB, aggregatePhase_, rhoA, rhoB, Yb)
    );

    const volScalarField nPf
    (
        IOobject::groupName("freeParticles", detachedPhase_.name()),
        nD(dP, detachedPhase_, rhoP, rhoP, detachedPhase_.Y(species_[0]))
    );

    const volScalarField RePrime
    (
        IOobject::groupName("RePrime", aggregatePhase_.name()),
        Re(dB, uPrimeB, nuL)
    );

    const volScalarField Bo
    (
        IOobject::groupName("Bo", aggregatePhase_.name()),
        Bond(dP, dB, rhoP, rhoL, epsL, interface.g(), interface.sigma())
    );

    const volScalarField tInd
    (IOobject::groupName("inductionTime", aggregatePhase_.name()),t(dP));

    Z_ =
        pos(dB - dBCrit)*pos(dP - dPCrit)
       *1.25*sqr(dP + dB)*sqrt(sqr(uPrimeP) + sqr(uPrimeB))
      + (1 - pos(dB - dBCrit)*pos(dP - dPCrit))
       *pow(0.5*(dP + dB), 3.0)*sqrt(pi*8.0/15.0*epsL/nuL);

    ZPrime_ = sqrt(C_)*cbrt(epsL)/sqr(cbrt(dB + dP));

    Pc_ = (1.5 + 4.0*pow(RePrime, 0.72)/15)*sqr(dP/dB);

    Pa_ =
        sqr
        (
            sin
            (
                2*atan
                (
                    exp
                    (
                      - ((45 + 8.0*pow(RePrime, 0.72))*uPrimeB*tInd)
                       /(15.0*dB*(dB/dP + 1.0))
                    )
                )
            )
        );

    Ps_ = 1.0 - exp(As_*(1.0 - 1.0/Bo));

    if (stability_ && detachment_)
    {
        Psi_ =
            (Z_*Pc_*Pa_*Ps_*nPf*nBa - ZPrime_*(1 - Ps_)*nBl)
           *rhoP*pi/6.0*pow3(dP);
    }
    else if (!stability_ && detachment_)
    {
        Psi_ =
            (Z_*Pc_*Pa_*nPf*nBa - ZPrime_*(1 - Ps_)*nBl)
           *rhoP*pi/6.0*pow3(dP);
    }
    else if (stability_ && !detachment_)
    {
        Psi_ = Z_*Pc_*Pa_*Ps_*nPf*nBa*rhoP*pi/6.0*pow3(dP);
    }
    else
    {
        Psi_ = Z_*Pc_*Pa_*nPf*nBa*rhoP*pi/6.0*pow3(dP);
    }

    Psi_ *=
        pos(detachedPhase_ - detachedPhase_.residualAlpha())
       *pos(aggregatePhase_ - residualAlpha_)
       *pos0
        (
            detachedPhase_*detachedPhase_.rho()
          - detachedPhase_.fluid().time().deltaT()*Psi_
        );

    result.set(species_[0], (sign_*Psi_).ptr());

    return result;
}


// ************************************************************************* //
