/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::phaseTransferModels::Castro

Description
    Entrainment model of Castro et al.

    The probability of finding a bubble of size $D$ at given depth \f$z\f$
    (Eq. 37) is not implemented, as computing the distance to the surface is a
    costly operation.

    References:
    \verbatim
        Castro, A. J., Li J., Carrica P. M. (2016).
        A mechanistic model of bubble entrainment in turbulent free surface
        flows.
        International Journal of Multiphase Flows.
    \endverbatim

Usage
    \table
        Property    | Description                          | Required | Default
        threshold   | Minimum entraining phase fraction    | no       | 0.2
        Sl0         | Bubble source intensity              | no       | 1
        N           | Number of integration steps          | no       | 10
    \endtable

SourceFiles
    Castro.C

\*---------------------------------------------------------------------------*/

#ifndef Castro_H
#define Castro_H

#include "entrainmentModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

namespace phaseTransferModels
{

/*---------------------------------------------------------------------------*\
                           Class Castro Declaration
\*---------------------------------------------------------------------------*/

class Castro
:
    public entrainmentModel
{
    // Private Data

        //- Bubble source intensity
        const dimensionedScalar Sl0_;

        //- Number of integration steps
        const scalar N_;

        //- Total entrainment flux of vortices
        mutable volScalarField J_;

        //- Update the entrainment rate once per time step
        mutable scalar rateUpdateTime_;


    // Private Member Functions

        //- Turbulent spectrum function fL (Eq 12)
        inline tmp<volScalarField> fL
        (
            const volScalarField& k,
            const volScalarField& eps,
            const dimensionedScalar l
        ) const;

        //- Vortex Froude number Frl
        inline tmp<volScalarField> Frl
        (
            const volScalarField& ul_l,
            const dimensionedScalar l
        ) const;

        //- Velocity of vortices of size l (Eq 14)
        inline tmp<volScalarField> ul
        (
            const volScalarField& eps,
            const volScalarField& fL_l,
            const dimensionedScalar l
        ) const;

        //- Hinze diameter (Eq 15)
        inline tmp<volScalarField> dHinze
        (
            const dispersedPhaseInterface& interface,
            const volScalarField& eps
        ) const;

        //- Onset depth (Eq 25)
        inline tmp<volScalarField> ze
        (
            const volScalarField& Frl_l,
            const dimensionedScalar l
        ) const;

        //- Probability of breakage (Eq. 30)
        inline tmp<volScalarField> PB
        (
            const dispersedPhaseInterface& interface,
            const volScalarField& ul_l,
            const dimensionedScalar l
        ) const;

        //- Single vortex entrainment source integrated over depth (Eq 40)
        inline tmp<volScalarField> IQ(const volScalarField& Frl_l) const;

        //- Entrainment volumetric flux from vortices of size l (Eq 42)
        inline tmp<volScalarField> Jl
        (
            const dispersedPhaseInterface& interface,
            const volScalarField& fL_l,
            const volScalarField& ul_l,
            const volScalarField& Frl_l,
            const volScalarField& ze_l,
            const dimensionedScalar l
        ) const;

        //- Log scale Simpson integration over vortex sizes (Eq 43)
        tmp<volScalarField> J
        (
            const dispersedPhaseInterface& interface,
            const volScalarField& k,
            const volScalarField& eps
        ) const;


protected:

    // Protected Member Functions

        //- Returns the mass transfer rate
        virtual tmp<volScalarField> rate(const phaseModel&) const;


public:

    //- Runtime type information
    TypeName("CastroEntrainment");


    // Constructors

        //- Construct from components
        Castro
        (
            const dictionary& dict,
            const phaseInterface& interface
        );


    //- Destructor
    virtual ~Castro();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace phaseTransferModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
