/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::phaseTransferModels::entrainmentModel

Description
    Abstract base class for entrainment models from a continuous phase to a
    disperse one. The convention is to use an ordered phase pair of the
    following structure: "(continuous in disperse)".

    The keyword 'threshold' is the minimum value of the entrainingContinuous
    phase volume fraction for entrainment to be active.

    The distribution of the computed entrainment rate between the different
    sizeGroups in case of polydisperse flow is handled in the population balance
    framework.

SourceFiles
    entrainmentModel.C

\*---------------------------------------------------------------------------*/

#ifndef entrainmentModel_H
#define entrainmentModel_H

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "phaseTransferModel.H"
#include "volFields.H"
#include "dictionary.H"
#include "runTimeSelectionTables.H"
#include "dispersedPhaseInterface.H"

namespace Foam
{

namespace phaseTransferModels
{

/*---------------------------------------------------------------------------*\
                      Class entrainmentModel Declaration
\*---------------------------------------------------------------------------*/

class entrainmentModel
:
    public phaseTransferModel
{
    // Private Typedefs

        typedef UPtrList<phaseModel> phaseModelPartialList;


    // Private Data

        //- Name of the continuous phases
        const phaseModelPartialList continuousPhases_;

        //- Threshold for volume fraction of entrainingContinuous phase
        const dimensionedScalar threshold_;


protected:

    // Protected Data

        //- Interface
        const dispersedPhaseInterface interface_;

        //- Continuous member of phase interface
        const phaseModel& pairContinuous_;

        //- Disperse member of phase interface
        const phaseModel& pairDisperse_;


    // Protected Member Functions

        //- Return the entrainment rate as computed by the model
        virtual tmp<volScalarField> rate(const phaseModel&) const = 0;


public:

    //- Runtime type information
    TypeName("entrainment");


    // Declare runtime construction

        declareRunTimeSelectionTable
        (
            autoPtr,
            entrainmentModel,
            dictionary,
            (
                const dictionary& dict,
                const phaseInterface& interface
            ),
            (dict, interface)
        );


    // Constructors

        //- Construct from a dictionary and an interface
        entrainmentModel
        (
            const dictionary& dict,
            const phaseInterface& interface
        );


    //- Destructor
    virtual ~entrainmentModel();


    // Member Functions

        //- Whether or not there is a mass transfer rate for the mixture
        virtual bool mixture() const;

        //- The mass transfer rate
        virtual tmp<volScalarField> dmdtf() const;

        //- The mass transfer rate derivative w.r.t pressure for the mixture
        virtual tmp<volScalarField> d2mdtdpf() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace phaseTransferModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
