/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::phaseTransferModels::detrainment

Description
    Morphology transfer model representing change from a dispersed phase to a
    continuous phase.

    The keyword 'pairContinuousRatioThreshold' is the minimum value of the
    ratio of the continuous phase volume fraction of the specified phasePair
    to the sum of all continuous phase volume fractions. It defines where
    detrainment is active; i.e. detrainment is active in cells with:

    \f[
        \frac{\alpha_{pairCont}}
             {\sum_{i \in cont} \alpha_{i}}
        >
        pairContinuousRatioThreshold
    \f]
    The value will limit the detrainment to take place only at on side
    of an interface (ratio > 0.X).

Usage
    \table
        Property    | Description                          | Required | Default
        nDeltaT     | Number of time steps for complete transfer   | no   | 100
        pairContinuousRatioThreshold | Minimum continuous fraction | no   | 0.9
    \endtable

SourceFiles
    detrainment.C

\*---------------------------------------------------------------------------*/

#ifndef detrainment_H
#define detrainment_H

#include "morphologyTransferModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

namespace phaseTransferModels
{

/*---------------------------------------------------------------------------*\
                         Class detrainment Declaration
\*---------------------------------------------------------------------------*/

class detrainment
:
    public morphologyTransferModel
{
    // Private Data

        //- Disperse member of phase pair
        const phaseModel& pairDisperse_;

        //- Continuous member of phase pair
        const phaseModel& pairContinuous_;

        //- Number of time steps to completely transfer volume fraction
        const scalar nDeltaT_;

        //- Threshold for volume fraction of pairContinuous phase
        const dimensionedScalar pairContinuousRatioThreshold_;


protected:

    // Protected Member Functions

        //- Return rate
        virtual tmp<volScalarField> rate(const phaseModel&) const;


public:

    //- Runtime type information
    TypeName("detrainment");


    // Constructors

        //- Construct from components
        detrainment
        (
            const dictionary& dict,
            const phaseInterface& interface
        );


    //- Destructor
    virtual ~detrainment();


    // Member Functions

        //- The mass transfer rate for the mixture
        virtual tmp<volScalarField> dmdtf() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace phaseTransferModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
