/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::potentialForceModel

Description
    Baseclass for force derived from a given potential field with a
    dimensionless proportional factor.

Usage
    Example entry in phaseProperties dictionary:
    \verbatim
    potentialForce
    {
        air_dispersedIn_water
        {
            type        constant;
            C           1;
        }
    }
    \endverbatim

    \table
        Property       | Description          | Required | Default
        potential      | Potential field name | no       | potential
        C              | Proportional factor  | yes      | none
    \endtable

See also
    Foam::MomentumTransferPhaseSystem

SourceFiles
    potentialForceModel.C
    potentialForceModelNew.C

\*---------------------------------------------------------------------------*/

#ifndef potentialForceModel_H
#define potentialForceModel_H

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "volFields.H"
#include "dictionary.H"
#include "dispersedPhaseInterface.H"
#include "runTimeSelectionTables.H"
#include "BlendedInterfacialModel.H"

namespace Foam
{

/*---------------------------------------------------------------------------*\
                Class potentialForceModel Declaration
\*---------------------------------------------------------------------------*/

class potentialForceModel
{
protected:

    // Protected Data

        //- Phase interface
        const dispersedPhaseInterface interface_;

        //- Pointer to potential field
        mutable autoPtr<volScalarField> potentialPtr_;

        //- PotentialForce field name
        const word potentialName_;

    // Protected Member Functions

        //- Return potentialForce coefficient
        virtual tmp<volScalarField> C() const = 0;

public:

    //- Runtime type information
    TypeName("potentialForceModel");


    // Declare runtime construction

        declareRunTimeSelectionTable
        (
            autoPtr,
            potentialForceModel,
            dictionary,
            (
                const dictionary& dict,
                const phaseInterface& interface
            ),
            (dict, interface)
        );


    // Static data members

        //- Force dimensions
        static const dimensionSet dimF;

        //- Does this model require correcting on fixed flux boundaries?
        static const bool correctFixedFluxBCs = true;


    // Constructors

        //- Construct from a dictionary and an interface
        potentialForceModel
        (
            const dictionary& dict,
            const phaseInterface& interface
        );


    //- Destructor
    virtual ~potentialForceModel();


    // Selectors

        static autoPtr<potentialForceModel> New
        (
            const dictionary& dict,
            const phaseInterface& interface,
            const bool outer=true
        );


    // Member Functions

        //- Return face potential force
        virtual tmp<surfaceScalarField> Ff() const;

        //- Get potential name
        Foam::word potentialName() const;
};


/*---------------------------------------------------------------------------*\
             Class blendedPotentialForceModel Declaration
\*---------------------------------------------------------------------------*/

class blendedPotentialForceModel
:
    public BlendedInterfacialModel<potentialForceModel>
{
public:

    // Constructors

        //- Inherit base class constructors
        using
            BlendedInterfacialModel<potentialForceModel>
          ::BlendedInterfacialModel;


    // Selectors

        static autoPtr<blendedPotentialForceModel> New
        (
            const dictionary& dict,
            const phaseInterface& interface
        );


    // Member Functions

        //- Return face potentialForce force
        tmp<surfaceScalarField> Ff() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
