/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "turbulentNormalStressPressureFvPatchScalarField.H"
#include "addToRunTimeSelectionTable.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "uniformDimensionedFields.H"
#include "phaseSystem.H"

// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

void Foam::turbulentNormalStressPressureFvPatchScalarField::
explicitTurbulentNormalStressRequired
(
    const dictionary& pimpleDict
)
{
    Switch active =
        pimpleDict.lookupOrDefault<Switch>
        (
            "explicitTurbulentNormalStress",
            false
        );

    if (!active)
    {
        FatalErrorInFunction
            << "Option explicitTurbulentNormalStress has to be turned on "
            << "in fvSolution.PIMPLE dictionary when using this boundary "
            << "condition."
            << exit(FatalError);
    }
}

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::turbulentNormalStressPressureFvPatchScalarField::
turbulentNormalStressPressureFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF
)
:
    fixedValueFvPatchScalarField(p, iF),
    pTNS_(p.size(), Zero)
{
    explicitTurbulentNormalStressRequired
    (
        iF.mesh().solution().dict().subDict("PIMPLE")
    );
}


Foam::turbulentNormalStressPressureFvPatchScalarField::
turbulentNormalStressPressureFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchScalarField(p, iF, dict, false),
    pTNS_("pTNS", dict, p.size())
{
    explicitTurbulentNormalStressRequired
    (
        iF.mesh().solution().dict().subDict("PIMPLE")
    );

    if (dict.found("value"))
    {
        fvPatchScalarField::operator=
        (
            scalarField("value", dict, p.size())
        );
    }
    else
    {
        fvPatchField<scalar>::operator=(pTNS_);
    }
}


Foam::turbulentNormalStressPressureFvPatchScalarField::
turbulentNormalStressPressureFvPatchScalarField
(
    const turbulentNormalStressPressureFvPatchScalarField& ptf,
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedValueFvPatchScalarField(ptf, p, iF, mapper),
    pTNS_(mapper(ptf.pTNS_))
{}


Foam::turbulentNormalStressPressureFvPatchScalarField::
turbulentNormalStressPressureFvPatchScalarField
(
    const turbulentNormalStressPressureFvPatchScalarField& ptf,
    const DimensionedField<scalar, volMesh>& iF
)
:
    fixedValueFvPatchScalarField(ptf, iF),
    pTNS_(ptf.pTNS_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::turbulentNormalStressPressureFvPatchScalarField::map
(
    const fvPatchScalarField& ptf,
    const fvPatchFieldMapper& mapper
)
{
    fixedValueFvPatchScalarField::map(ptf, mapper);

    const turbulentNormalStressPressureFvPatchScalarField& tiptf =
        refCast<const turbulentNormalStressPressureFvPatchScalarField>(ptf);

    mapper(pTNS_, tiptf.pTNS_);
}


void Foam::turbulentNormalStressPressureFvPatchScalarField::updateCoeffs()
{
    if (updated())
    {
        return;
    }

    // Lookup the fluid model
    const phaseSystem& fluid =
    (
        db().lookupObject<phaseSystem>(phaseSystem::propertiesName)
    );

    // Turbulent normal stress term
    scalarField TNS(pTNS_.size(), Zero);

    forAll(fluid.phases(), phasei)
    {
        const phaseModel& phase = fluid.phases()[phasei];

        const scalarField& alphap =
            patch().lookupPatchField<volScalarField, scalar>
            (
                IOobject::groupName("alpha", phase.name())
            );

        const scalarField& rhoAlphap =
            patch().lookupPatchField<volScalarField, scalar>
            (
                IOobject::groupName("rho", phase.name())
            );

        if
        (
            db().foundObject<volScalarField>
            (
                IOobject::groupName("k", phase.name())
            )
        )
        {
            const scalarField& kp =
                patch().lookupPatchField<volScalarField, scalar>
                (
                    IOobject::groupName("k", phase.name())
                );

            TNS += 2./3.*alphap*rhoAlphap*kp;
        }
    }

    operator==(pTNS_ - TNS);

    fixedValueFvPatchScalarField::updateCoeffs();
}


void Foam::turbulentNormalStressPressureFvPatchScalarField::write
(
    Ostream& os
) const
{
    fvPatchScalarField::write(os);
    writeEntry(os, "pTNS", pTNS_);
    writeEntry(os, "value", *this);
}


// * * * * * * * * * * * * * * Build Macro Function  * * * * * * * * * * * * //

namespace Foam
{
    makePatchTypeField
    (
        fvPatchScalarField,
        turbulentNormalStressPressureFvPatchScalarField
    );
}

// ************************************************************************* //
