/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "wallBoiling.H"
#include "addToRunTimeSelectionTable.H"
#include "alphatWallBoilingWallFunctionFvPatchScalarField.H"
#include "phaseSystem.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace diameterModels
{
namespace nucleationModels
{
    defineTypeNameAndDebug(wallBoiling, 0);
    addToRunTimeSelectionTable
    (
        nucleationModel,
        wallBoiling,
        dictionary
    );
}
}
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::diameterModels::nucleationModels::wallBoiling::
wallBoiling
(
    const populationBalanceModel& popBal,
    const dictionary& dict
)
:
    nucleationModel(popBal, dict)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::diameterModels::nucleationModels::wallBoiling::precompute()
{
    const volScalarField& alphat =
        popBal_.mesh().lookupObject<volScalarField>
        (
            IOobject::groupName("alphat", popBal_.continuousPhase().name())
        );

    const volScalarField::Boundary& alphatBf = alphat.boundaryField();

    typedef compressible::alphatWallBoilingWallFunctionFvPatchScalarField
        alphatWallBoilingWallFunction;

    forAll(alphatBf, patchi)
    {
        if (isA<alphatWallBoilingWallFunction>(alphatBf[patchi]))
        {
            const alphatWallBoilingWallFunction& alphatw =
                refCast<const alphatWallBoilingWallFunction>(alphatBf[patchi]);

            const scalarField& dDep = alphatw.dDeparture();

            if (min(dDep) < velGroup_.sizeGroups().first().dSph().value())
            {
                Warning
                    << "Minimum departure diameter " << min(dDep)
                    << " m outside of range ["
                    << velGroup_.sizeGroups().first().dSph().value() << ", "
                    << velGroup_.sizeGroups().last().dSph().value() << "] m"
                    << " at patch " << alphatw.patch().name()
                    << endl
                    << "    The nucleation rate in populationBalance "
                    << popBal_.name() << " is set to zero." << endl
                    << "    Adjust discretisation over property space to"
                    << " suppress this warning."
                    << endl;
            }
            else if (max(dDep) > velGroup_.sizeGroups().last().dSph().value())
            {
                Warning
                    << "Maximum departure diameter " << max(dDep)
                    << " m outside of range ["
                    << velGroup_.sizeGroups().first().dSph().value() << ", "
                    << velGroup_.sizeGroups().last().dSph().value() << "] m"
                    << " at patch " << alphatw.patch().name()
                    << endl
                    << "    The nucleation rate in populationBalance "
                    << popBal_.name() << " is set to zero." << endl
                    << "    Adjust discretisation over property space to"
                    << " suppress this warning."
                    << endl;
            }
        }
    }
}


void
Foam::diameterModels::nucleationModels::wallBoiling::addToNucleationRate
(
    volScalarField& nucleationRate,
    const label i
)
{
    const sizeGroup& fi = popBal_.sizeGroups()[i];
    const phaseModel& phase = fi.phase();
    const volScalarField& rho = phase.rho();

    const volScalarField& alphat =
        popBal_.mesh().lookupObject<volScalarField>
        (
            IOobject::groupName("alphat", popBal_.continuousPhase().name())
        );

    const volScalarField::Boundary& alphatBf = alphat.boundaryField();

    typedef compressible::alphatWallBoilingWallFunctionFvPatchScalarField
        alphatWallBoilingWallFunction;

    forAll(alphatBf, patchi)
    {
        if
        (
            isA<alphatWallBoilingWallFunction>(alphatBf[patchi])
        )
        {
            const alphatWallBoilingWallFunction& alphatw =
                refCast<const alphatWallBoilingWallFunction>(alphatBf[patchi]);

            const scalarField& dmdt = alphatw.dmdtf();
            const scalarField& dDep = alphatw.dDeparture();

            const labelList& faceCells = alphatw.patch().faceCells();

            dimensionedScalar unitLength("unitLength", dimLength, 1);

            forAll(alphatw, facei)
            {
                if (dmdt[facei] > small)
                {
                    const label faceCelli = faceCells[facei];

                    nucleationRate[faceCelli] +=
                        popBal_.eta
                        (
                            i,
                            fi.x()/pow3(fi.dSph())*pow3(dDep[facei]*unitLength)
                        ).value()
                       *dmdt[facei]/rho[faceCelli]/fi.x().value();
                }
            }
        }
    }
}


// ************************************************************************* //
