/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::wallBoilingModels::departureDiameterModels::ShinJonesDepartureDiameter

Description
    Shin & Jones (1993) correlation for bubble departure diameter.

    References:
    \verbatim
        Shin, T. S., & Jones, O. C. (1993).
        Nucleation and flashing in nozzles - 1: A distributed nucleation model.
        International Journal of Multiphase FLow, 19(6), 943-964.
    \endverbatim

SourceFiles
    ShinJonesDepartureDiameter.C

\*---------------------------------------------------------------------------*/

#ifndef ShinJonesDepartureDiameter_H
#define ShinJonesDepartureDiameter_H

#include "departureDiameterModel.H"
#include "uniformDimensionedFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace wallBoilingModels
{
namespace departureDiameterModels
{

/*---------------------------------------------------------------------------*\
                 Class ShinJonesDepartureDiameter Declaration
\*---------------------------------------------------------------------------*/

class ShinJonesDepartureDiameter
:
    public departureDiameterModel
{
    // Private Data

        //- Optional phase name
        const word phaseName_;

        //- User adjustable coefficient for departure diameter
        const dimensionedScalar K_;

        //- Minimum departure diameter
        const dimensionedScalar dMin_;


    // Private Member Functions

        //- Calculate and return the bubble departure diameter
        template<class ScalarFieldType>
        tmp<ScalarFieldType> calculate
        (
            const ScalarFieldType& Tl,
            const ScalarFieldType& Tsatw,
            const ScalarFieldType& L,
            const ScalarFieldType& rhoLiquid,
            const ScalarFieldType& rhoVapour,
            const ScalarFieldType& muLiquid,
            const ScalarFieldType& nuLiquid,
            const ScalarFieldType& nuVapour,
            const ScalarFieldType& sigma,
            const ScalarFieldType& magTau
        ) const;


public:

    //- Runtime type information
    TypeName("ShinJones");


    // Constructors

        //- Construct from a dictionary
        ShinJonesDepartureDiameter(const dictionary& dict);

        //- Copy construct
        ShinJonesDepartureDiameter
        (
            const ShinJonesDepartureDiameter& model
        );

        //- Construct and return a clone
        virtual autoPtr<departureDiameterModel> clone() const
        {
            return autoPtr<departureDiameterModel>
            (
                new ShinJonesDepartureDiameter(*this)
            );
        }


    //- Destructor
    virtual ~ShinJonesDepartureDiameter();


    // Member Functions

        //- Calculate and return the departure diameter field for a wall patch
        virtual tmp<scalarField> dDeparture
        (
            const phaseModel& liquid,
            const phaseModel& vapour,
            const label patchi,
            const scalarField& Tl,
            const scalarField& Tsatw,
            const scalarField& L
        ) const;

        //- Calculate and return the departure diameter field for a stationary
        //  phase and fluid interface
        virtual tmp<volScalarField> dDeparture
        (
            const phaseModel& liquid,
            const phaseModel& vapour,
            const phaseModel& solid,
            const volScalarField& Tf,
            const volScalarField& Tsatw,
            const volScalarField& L
        ) const;

        //- Write to stream
        virtual void write(Ostream& os) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace departureDiameterModels
} // End namespace wallBoilingModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
