/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::InducedTurbulencePhaseSystem

Description
    Class which provides infrastructure for induced turbulence.

    Note: This class has to instantiated after momentumTransferPhaseSystem

SourceFiles
    InducedTurbulencePhaseSystem.C

\*---------------------------------------------------------------------------*/

#ifndef InducedTurbulencePhaseSystem_H
#define InducedTurbulencePhaseSystem_H

#include "phaseSystem.H"
#include "HashPtrTable.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

class blendingMethod;
class blendedInducedTurbulenceModel;

/*---------------------------------------------------------------------------*\
                Class InducedTurbulencePhaseSystem Declaration
\*---------------------------------------------------------------------------*/

template<class BasePhaseSystem>
class InducedTurbulencePhaseSystem
:
    public BasePhaseSystem
{
protected:

    // Protected Typedefs

        typedef HashPtrTable
        <
            volScalarField,
            phaseInterfaceKey,
            phaseInterfaceKey::hash
        >
        KkTable;

        typedef HashPtrTable
        <
            volScalarField,
            phaseInterfaceKey,
            phaseInterfaceKey::hash
        >
        KepsTable;

        typedef HashTable
        <
            autoPtr<blendedInducedTurbulenceModel>,
            phaseInterfaceKey,
            phaseInterfaceKey::hash
        > inducedTurbulenceModelTable;


private:

    // Private Data

        //- Turbulent kinetic energy source terms
        KkTable Kks_;

        //- Turbulent dissipation rate source terms
        KepsTable Kepss_;


        // Sub Models

            //- Induced turbulence models
            inducedTurbulenceModelTable inducedTurbulenceModels_;


public:

    // Constructors

        //- Construct from fvMesh
        InducedTurbulencePhaseSystem(const fvMesh&);


    //- Destructor
    virtual ~InducedTurbulencePhaseSystem();


    // Member Functions

        //- Wrapper to return momentum transfer matrices from
        //  momentumTransferPhaseSystem and to update the tables for the
        //  induced turbulence source terms
        virtual autoPtr<phaseSystem::momentumTransferTable> momentumTransfer();

        //- Turbulent kinetic energy source for phase
        virtual tmp<volScalarField> Kk(const phaseModel& phase) const;

        //- Turbulent kinetic energy source tensor for phase
        virtual tmp<volSymmTensorField> tensorKk(const phaseModel& phase) const;

        //- Turbulent dissipation rate source for phase
        virtual tmp<volScalarField> Keps(const phaseModel& phase) const;

        //- Read base phaseProperties dictionary
        virtual bool read();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
    #include "InducedTurbulencePhaseSystem.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
