/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "ResolvedPhaseSystem.H"

#include "interfaceCompressionModel.H"
#include "surfaceTensionForceModel.H"
#include "subGridSurfaceTensionModel.H"
#include "interfaceIndicatorMethod.H"
#include "underResolutionModel.H"
#include "clusterTrackingModel.H"
#include "moveAlphaByPotentialModel.H"

#include "segregatedPhaseInterface.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class BasePhaseSystem>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::ResolvedPhaseSystem
(
    const fvMesh& mesh
)
:
    BasePhaseSystem(mesh)
{
    this->generateInterfacialModels(interfaceCompressionModels_);

    surfaceTensionForceModel_ = surfaceTensionForceModel::New(*this);

    if (this->found("subGridSurfaceTension"))
    {
        this->generateInterfacialModels(subGridSurfaceTensionModels_);
    }

    interfaceIndicatorMethod_ = interfaceIndicatorMethod::New(*this);

    if (this->found("underResolution"))
    {
        this->generateInterfacialModels(underResolutionModels_);
    }

    if (this->found("clusterTracking"))
    {
        this->generateInterfacialModels(clusterTrackingModels_);
    }

    if (this->found("moveAlphaByPotential"))
    {
        this->generateInterfacialModels(moveAlphaByPotentialModels_);
    }
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

template<class BasePhaseSystem>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::~ResolvedPhaseSystem()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

template<class BasePhaseSystem>
Foam::tmp<Foam::surfaceScalarField>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::phir
(
    const phaseModel& phase1,
    const phaseModel& phase2
) const
{
    tmp<surfaceScalarField> phir(BasePhaseSystem::phir(phase1, phase2));

    const phaseInterfaceKey key(phase1, phase2);

    if (interfaceCompressionModels_.found(key))
    {
        if (interfaceCompressionModels_[key]().type() != "none")
        {
            phir.ref() += interfaceCompressionModels_[key]().phirc(phase1);
        }
    }

    if (moveAlphaByPotentialModels_.found(key))
    {
        if (moveAlphaByPotentialModels_[key]().type() != "none")
        {
            phir.ref() +=
                moveAlphaByPotentialModels_[key]().phirc(phase1);
        }
    }

    return phir;
}


template<class BasePhaseSystem>
void Foam::ResolvedPhaseSystem<BasePhaseSystem>::addCompressionFlux
(
    const phaseModel& phase,
    surfaceScalarField& alphaPhi
) const
{
    BasePhaseSystem::addCompressionFlux(phase, alphaPhi);

    if (!interfaceCompressionModels_.empty())
    {
        const volScalarField& alpha = phase;

        forAll(this->phases(), phasei)
        {
            const phaseModel& phase2 = this->phases()[phasei];
            const volScalarField& alpha2 = phase2;
            const phaseInterfaceKey key(phase, phase2);

            if (&phase2 == &phase) continue;

            interfaceCompressionModelTable::const_iterator cAlpha
            (
                interfaceCompressionModels_.find(key)
            );

            if (cAlpha != interfaceCompressionModels_.end())
            {
                const surfaceScalarField phir
                (
                    interfaceCompressionModels_[key]().phirc(phase)
                );

                const word phirScheme
                (
                    "div(phir,"
                  + alpha2.name() + ',' + alpha.name()
                  + ')'
                );

                alphaPhi += fvc::flux
                (
                    -fvc::flux(-phir, alpha2, phirScheme),
                    alpha,
                    phirScheme
                );
            }
        }
    }

    if (moveAlphaByPotentialModels_.size())
    {
        const volScalarField& alpha = phase;

        forAll(this->phases(), phasei)
        {
            const phaseModel& phase2 = this->phases()[phasei];
            const volScalarField& alpha2 = phase2;
            const phaseInterfaceKey key(phase, phase2);

            if (&phase2 == &phase) continue;

            moveAlphaByPotentialModelTable::const_iterator cAlpha
            (
                moveAlphaByPotentialModels_.find(key)
            );

            if (cAlpha != moveAlphaByPotentialModels_.end())
            {
                const surfaceScalarField phir
                (
                    moveAlphaByPotentialModels_[key]().phirc(phase)
                );

                const word phirScheme
                (
                    "div(phir,"
                  + alpha2.name() + ',' + alpha.name()
                  + ')'
                );

                alphaPhi += fvc::flux
                (
                    -fvc::flux(-phir, alpha2, phirScheme),
                    alpha,
                    phirScheme
                );
            }
        }
    }
}


template<class BasePhaseSystem>
Foam::tmp<Foam::surfaceScalarField>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::surfaceTension
(
    const phaseModel& phase1
) const
{
    tmp<surfaceScalarField> tSurfaceTension
    (
        new surfaceScalarField
        (
            IOobject
            (
                "surfaceTension",
                this->mesh().time().name(),
                this->mesh()
            ),
            this->mesh(),
            dimensionedScalar
            (
                "surfaceTension",
                dimensionSet(1, -2, -2, 0, 0),
                0
            )
        )
    );

    forAll(this->phases(), phasej)
    {
        const phaseModel& phase2 = this->phases()[phasej];

        if (&phase2 == &phase1) continue;

        const phaseInterfaceKey key(phase1, phase2);

        if (interfaceCompressionModels_.found(key))
        {
            if (interfaceCompressionModels_[key]().type() != "none")
            {
                const segregatedPhaseInterface interface(phase1, phase2);

                tSurfaceTension.ref() +=
                    surfaceTensionForceModel_->surfaceTensionForcef(interface);

                if (subGridSurfaceTensionModels_.found(key))
                {
                    if
                    (
                        subGridSurfaceTensionModels_[key]().type()
                        != "none"
                    )
                    {
                        tSurfaceTension.ref() +=
                            subGridSurfaceTensionModels_[key]().
                            subGridSurfaceTensionForcef();
                    }
                }
            }
        }
    }

    return tSurfaceTension;
}


template<class BasePhaseSystem>
Foam::tmp<Foam::volScalarField>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::interfaceIndicator
(
    const phaseModel& phase
) const
{
   return interfaceIndicatorMethod_->interfaceIndicator(phase);
}


template<class BasePhaseSystem>
Foam::tmp<Foam::volVectorField> Foam::ResolvedPhaseSystem<BasePhaseSystem>::UrI
(
    const phaseInterfaceKey& key
) const
{
    if (underResolutionModels_.found(key))
    {
        return underResolutionModels_[key]().UrI();
    }
    else
    {
        FatalErrorInFunction
            << "Requesting UrI from phase interface "
            << phaseInterface(*this, key).name()
            << ", underResolutionModel not found." << endl;

        return tmp<volVectorField>(nullptr);
    }
}


template<class BasePhaseSystem>
Foam::tmp<Foam::volScalarField>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::underResolution
(
    const phaseInterfaceKey& key
) const
{
    if (underResolutionModels_.found(key))
    {
        return underResolutionModels_[key]().underResolution();
    }
    else
    {
        FatalErrorInFunction
            << "Requesting underResolution from phase interface "
            << phaseInterface(*this, key).name()
            << ", underResolutionModel not found." << endl;

        return tmp<volScalarField>(nullptr);
    }
}


template<class BasePhaseSystem>
void Foam::ResolvedPhaseSystem<BasePhaseSystem>::solve
(
    const PtrList<volScalarField>& rAUs,
    const PtrList<surfaceScalarField>& rAUfs
)
{
    // Conversion of large fluid particles to resolved state
    if (clusterTrackingModels_.size())
    {
        // Count active cluster tracking pairs
        label activeTrackings(0);
        forAllIter
        (
            clusterTrackingModelTable,
            clusterTrackingModels_,
            clusterTrackingModelIter
        )
        {
            activeTrackings += clusterTrackingModelIter()->isActive();
        }

        // Process one at a time
        if (activeTrackings == 1)
        {
            forAllIter
            (
                clusterTrackingModelTable,
                clusterTrackingModels_,
                clusterTrackingModelIter
            )
            {
                if (clusterTrackingModelIter()->isActive())
                {
                    clusterTrackingModelIter()->updateClusters();
                }
            }
        }
        else // Identify phase pair with largest over-resolved diameter
        {
            Info<< "Cluster tracking: Find candidate" << endl;
            scalar largestDiameter(0);
            phaseInterfaceKey candidateKey;

            forAllIter
            (
                clusterTrackingModelTable,
                clusterTrackingModels_,
                clusterTrackingModelIter
            )
            {
                // Update all clustering pairs and find largest diameter among
                // active ones
                if
                (
                    clusterTrackingModelIter()->updateClusters()
                && clusterTrackingModelIter()->diameter() > largestDiameter
                )
                {
                    largestDiameter = clusterTrackingModelIter()->diameter();
                    candidateKey = clusterTrackingModelIter()->key();
                }
            }

            // Keep only identified phase pair active
            forAllIter
            (
                clusterTrackingModelTable,
                clusterTrackingModels_,
                clusterTrackingModelIter
            )
            {
                if
                (
                    largestDiameter
                && (clusterTrackingModelIter()->key() == candidateKey)
                )
                {
                    Info<< "Cluster tracking: Keep "
                        << clusterTrackingModelIter()->compressedPhase().name()
                        << endl;
                }
                else
                {
                    clusterTrackingModelIter()->discard();
                }
            }
        }
    }

    BasePhaseSystem::solve(rAUs, rAUfs);
}


template<class BasePhaseSystem>
Foam::tmp<Foam::volScalarField>
Foam::ResolvedPhaseSystem<BasePhaseSystem>::K
(
    const phaseInterfaceKey& key
) const
{
    const phaseInterface interface(*this, key);

    return surfaceTensionForceModel_->K
    (
        segregatedPhaseInterface(interface.phase1(), interface.phase2())
    );
}


template<class BasePhaseSystem>
void Foam::ResolvedPhaseSystem<BasePhaseSystem>::correct()
{
    BasePhaseSystem::correct();

    forAllIter
    (
        underResolutionModelTable,
        underResolutionModels_,
        underResolutionModelIter
    )
    {
        underResolutionModelIter()->correct();
    }
}


template<class BasePhaseSystem>
bool Foam::ResolvedPhaseSystem<BasePhaseSystem>::read()
{
    if (BasePhaseSystem::read())
    {
        bool readOK = true;

        // Read models ...

        return readOK;
    }
    else
    {
        return false;
    }
}


// ************************************************************************* //
