/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::ResolvedPhaseSystem

Description
    Class which adds functionality for modeling resolved interfaces such as
    interface compression, surface tension and contact angle correction.

SourceFiles
    ResolvedPhaseSystem.C

\*---------------------------------------------------------------------------*/

#ifndef ResolvedPhaseSystem_H
#define ResolvedPhaseSystem_H

#include "phaseSystem.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

class interfaceCompressionModel;
class surfaceTensionForceModel;
class subGridSurfaceTensionModel;
class interfaceIndicatorMethod;
class underResolutionModel;
class clusterTrackingModel;
class moveAlphaByPotentialModel;
class blendedPotentialForceModel;

/*---------------------------------------------------------------------------*\
                     Class ResolvedPhaseSystem Declaration
\*---------------------------------------------------------------------------*/

template<class BasePhaseSystem>
class ResolvedPhaseSystem
:
    public BasePhaseSystem
{
private:

    // Private Typedefs

        typedef HashTable
        <
            autoPtr<interfaceCompressionModel>,
            phaseInterfaceKey,
            phaseInterfaceKey::hash
        > interfaceCompressionModelTable;

        typedef HashTable
        <
            autoPtr<subGridSurfaceTensionModel>,
            phaseInterfaceKey,
            phaseInterfaceKey::hash
        > subGridSurfaceTensionModelTable;

        typedef HashTable
        <
            autoPtr<underResolutionModel>,
            phaseInterfaceKey,
            phaseInterfaceKey::hash
        > underResolutionModelTable;

        typedef HashTable
        <
            autoPtr<clusterTrackingModel>,
            phaseInterfaceKey,
            phaseInterfaceKey::hash
        > clusterTrackingModelTable;

        typedef HashTable
        <
            autoPtr<moveAlphaByPotentialModel>,
            phaseInterfaceKey,
            phaseInterfaceKey::hash
        > moveAlphaByPotentialModelTable;

    // Private Data

        //- Interface compression models
        interfaceCompressionModelTable interfaceCompressionModels_;

        //- Surface tension force models
        autoPtr<surfaceTensionForceModel> surfaceTensionForceModel_;

        //- Sub grid surface tension models
        subGridSurfaceTensionModelTable subGridSurfaceTensionModels_;

        //- Resolved interface detection method
        autoPtr<interfaceIndicatorMethod> interfaceIndicatorMethod_;

        //- Under resolution models
        underResolutionModelTable underResolutionModels_;

        //- Models for cluster tracking
        clusterTrackingModelTable clusterTrackingModels_;

        //- Models for moving alpha by potential
        moveAlphaByPotentialModelTable moveAlphaByPotentialModels_;

public:

    // Constructors

        //- Construct from fvMesh
        ResolvedPhaseSystem(const fvMesh&);


    //- Destructor
    virtual ~ResolvedPhaseSystem();


    // Member Functions

        //- Return relative flux for interface compression
        virtual tmp<surfaceScalarField> phir
        (
            const phaseModel& phase1,
            const phaseModel& phase2
        ) const;

        //- Add interface compression flux
        virtual void addCompressionFlux
        (
            const phaseModel& phase,
            surfaceScalarField& alphaPhi
        ) const;

        //- Return the surface tension force
        tmp<surfaceScalarField> surfaceTension(const phaseModel& phase) const;

        //- Indicator field of the proximity of a resolved phase interface
        //  Values are distributed from 0 (far) to 1 (near).
        virtual tmp<volScalarField> interfaceIndicator
        (
            const phaseModel& phase
        ) const;

        //- Relative velocity across the interface
        virtual tmp<volVectorField> UrI(const phaseInterfaceKey& key) const;

        //- Degree of under resolution due to shear and curvature for phase pair
        virtual tmp<volScalarField> underResolution
        (
            const phaseInterfaceKey& key
        ) const;

        //- Cluster tracking
        virtual void solve
        (
            const PtrList<volScalarField>& rAUs,
            const PtrList<surfaceScalarField>& rAUfs
        );

        //- Interface curvature for phase pair
        tmp<volScalarField> K(const phaseInterfaceKey& key) const;

        //- Correct derived properties
        virtual void correct();

        //- Read base phaseProperties dictionary
        virtual bool read();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
    #include "ResolvedPhaseSystem.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
