/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "addToRunTimeSelectionTable.H"
#include "fvcGrad.H"
#include "interfaceIndicatorMethod.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    defineTypeNameAndDebug(interfaceIndicatorMethod, 0);
    defineRunTimeSelectionTable(interfaceIndicatorMethod, dictionary);
}


// * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * * //

//- Provides default implementation for pure virtual function
Foam::tmp<Foam::volScalarField>
Foam::interfaceIndicatorMethod::interfaceIndicator
(
    const phaseModel& phase1,
    const phaseModel& phase2
) const
{
    NotImplemented;
    return tmp<volScalarField>(nullptr);
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::interfaceIndicatorMethod::interfaceIndicatorMethod
(
    const phaseSystem& fluid
)
:
    fluid_(fluid)
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::interfaceIndicatorMethod::~interfaceIndicatorMethod()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::interfaceIndicatorMethod::interfaceIndicator
(
    const phaseModel& phase
) const
{
    // Consider only interfaces between continuous phases
    if (!phase.continuous())
    {
        FatalErrorInFunction
            << "Attempt to detect interface for non-continuous phase: "
            << phase.name() << endl
            << exit(FatalError);
    }
    const phaseSystem::phaseModelPartialList& phases
    (
        fluid_.continuousPhases()
    );

    tmp<volScalarField> ri
    (
        new volScalarField
        (
            IOobject
            (
                "tmp",
                fluid_.mesh().time().name(),
                fluid_.mesh()
            ),
            fluid_.mesh(),
            dimensionedScalar(dimless, Zero)
        )
    );

    // Use maximum pair contribution
    forAll(phases, phasei)
    {
        const phaseModel& phase2 = phases[phasei];

        if (phase2 != phase)
        {
            ri.ref() = max(interfaceIndicator(phase, phase2), ri());
        }
    }

    // Allow for caching after evaluation
    ri.ref().rename(IOobject::groupName("interfaceIndicator", phase.name()));

    return ri;
}


// ************************************************************************* //
