/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "interfaceIndicatorMethod.H"
#include "phaseSystem.H"

// Helper class to handle error reporting
namespace Foam
{
namespace interfaceIndicatorMethods
{

class undefined
:  public interfaceIndicatorMethod
{
protected:

    //- Handles error reporting for missing dict entry in phaseProperties.
    virtual tmp<volScalarField> interfaceIndicator
    (
        const phaseModel& phase1, const phaseModel& phase2
    ) const
    {
        // Use dictionary error handler
        const word selectedType
        (
            fluid_.subDict(interfaceIndicatorMethod::typeName).lookup("type")
        );

        return interfaceIndicatorMethod::interfaceIndicator(phase1, phase2);
    }


public:

    //- Inherit all constructors from base
    using interfaceIndicatorMethod::interfaceIndicatorMethod;
};

}
}

// * * * * * * * * * * * * * * * * Selector  * * * * * * * * * * * * * * * * //

Foam::autoPtr<Foam::interfaceIndicatorMethod>
Foam::interfaceIndicatorMethod::New
(
    const phaseSystem& fluid
)
{
    // Select the base method if method information is missing.
    const word methodDictName = Foam::interfaceIndicatorMethod::typeName;
    const word typeKeyword = "type";
    word selectedType;

    if (fluid.found(methodDictName))
    {
        const dictionary& methodDict = fluid.subDict(methodDictName);

        if (methodDict.found(typeKeyword))
        {
            selectedType = methodDict.lookup<word>(typeKeyword);
            Info<< "Selecting interfaceIndicatorMethod: "
                << selectedType << endl;

            dictionaryConstructorTable::iterator cstrIter =
                dictionaryConstructorTablePtr_->find(selectedType);

            if (cstrIter == dictionaryConstructorTablePtr_->end())
            {
                FatalErrorInFunction
                    << "Unknown " << methodDictName << " type "
                    << selectedType << endl << endl
                    << "Valid " << methodDictName << " types are : " << endl
                    << dictionaryConstructorTablePtr_->sortedToc() << endl
                    << exit(FatalError);
            }

            return cstrIter()(fluid);
        }
    }

    // No user input - select the undefined method which lazily reports errors
    // if client model code requests a member function of base method.
    return
        autoPtr<interfaceIndicatorMethod>
        (
            new interfaceIndicatorMethods::undefined(fluid)
        );
}
