/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::interfaceIndicatorMethods::useBlendingMethod

Description
    Determine the interface indicator field for a pair of continuous phases,
    i and j:

    \f[
        \mathrm{RI}_{ij} = (1 - f_1 - f_2)_{ij},
    \f]

    where \f$f_1\f$ and \f$f_2\f$ are provided by the selected blendingMethod.

Usage
    Example setup for continuous air and water phases:
    \verbatim
    interfaceIndicatorMethod
    {
        type    useBlendingMethod;
        blendingMethod
        {
            type                psiSurf;
            critAlpha.air       0.3;
            critAlpha.water     0.3;
            deltaAlpha.air      0.05;
            deltaAlpha.water    0.05;
            f                   0.1;
            n                   5;
        }
    }
    \endverbatim

See also
    Foam::blendingMethod
    Foam::blendingMethods::psiSurf

SourceFiles
    useBlendingMethod.C

\*---------------------------------------------------------------------------*/

#ifndef useBlendingMethod_H
#define useBlendingMethod_H

#include "blendingMethod.H"
#include "interfaceIndicatorMethod.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace interfaceIndicatorMethods
{

/*---------------------------------------------------------------------------*\
                      Class useBlendingMethod Declaration
\*---------------------------------------------------------------------------*/

class useBlendingMethod
:
    public interfaceIndicatorMethod
{
    // Private Data

        //- Blending method used for interface detection
        autoPtr<blendingMethod> blendingMethod_;


protected:

    // Protected Member Functions

        //- Indicator field between a phase pair
        virtual tmp<volScalarField> interfaceIndicator
        (
            const phaseModel& phase1,
            const phaseModel& phase2
        ) const;


public:

    //- Runtime type information
    TypeName("useBlendingMethod");


    // Constructors

        //- Construct from a dictionary from phaseSystem
        useBlendingMethod
        (
            const phaseSystem& fluid
        );


    //- Destructor
    virtual ~useBlendingMethod();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace interfaceIndicatorMethods
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
