/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "moveAlphaByPotentialModel.H"
#include "phaseSystem.H"
#include "fvcGrad.H"
#include "surfaceInterpolate.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    defineTypeNameAndDebug(moveAlphaByPotentialModel, 0);
    defineRunTimeSelectionTable(moveAlphaByPotentialModel, dictionary);
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::moveAlphaByPotentialModel::moveAlphaByPotentialModel
(
    const dictionary& dict,
    const phaseInterface& interface
)
:
    transportedPhase_
    (
        interface.mesh().lookupObject<phaseModel>
        (
            IOobject::groupName("alpha", dict.lookup("transportedPhase"))
        )
    ),
    displacedPhase_
    (
        interface.mesh().lookupObject<phaseModel>
        (
            IOobject::groupName("alpha", dict.lookup("displacedPhase"))
        )
    ),
    potentialPtr_(nullptr),
    potentialName_(dict.lookupOrDefault<word>("potential", "potential")),
    maskName_(dict.lookupOrDefault<word>("mask", "mask")),
    mesh_(interface.mesh())
{
    // If not already registered, read potential from file
    if (!mesh_.foundObject<volScalarField>(potentialName_))
    {
        Info<< "    Reading field " << potentialName_ << endl;
        potentialPtr_.reset
        (
            new volScalarField
            (
                IOobject
                (
                    potentialName_,
                    mesh_.time().name(),
                    mesh_,
                    IOobject::MUST_READ,
                    IOobject::NO_WRITE
                ),
                mesh_
            )
        );
    }
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::moveAlphaByPotentialModel::~moveAlphaByPotentialModel()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Foam::tmp<Foam::surfaceScalarField> Foam::moveAlphaByPotentialModel::phirc
(
    const phaseModel& phase
) const
{
    const phaseSystem& fluid(phase.fluid());

    const Foam::volScalarField& potential
    (
         potentialPtr_.valid()
       ? potentialPtr_()
       : mesh_.lookupObject<volScalarField>(potentialName_)
    );

    Foam::surfaceScalarField normGradPotential
    (
        fluid.nHatf(potential, 1 - potential)
    );

    if (mesh_.foundObject<volScalarField>(maskName_))
    {
        const Foam::volScalarField& mask
        (
            mesh_.lookupObject<volScalarField>(maskName_)
        );

        // Apply mask so that 1 is only realised on faces within the region
        normGradPotential *= pos0(fvc::interpolate(mask) - 1);
    }

    scalar sign = 1;
    if (&phase == &transportedPhase_)
    {
        sign = 1;
    }
    else if (&phase == &displacedPhase_)
    {
        sign = -1;
    }
    else
    {
        FatalErrorInFunction
            << "The specified phase, " << phase.name() << ", is not in "
            << "the pair " << transportedPhase_.name() << ", "
            << displacedPhase_.name()
            << exit(FatalError);
    }

    return sign*C()*normGradPotential;
}


// ************************************************************************* //
