/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::moveAlphaByPotentialModel

Description
    Base class to transport a specific phase through another phase. The
    transport velocity is defined by the user and the transport direction is
    computed from the normalised gradient field of a given potential field. The
    displaced phase is transported in the opposite direction.

    The transport is realised by an additional flux in the phase transport
    equations. The transport can be limited to certain regions, defined by an
    additional mask field.

Usage
    Example entry in phaseProperties dictionary:
    \verbatim
    moveAlphaByPotential
    {
        airD_waterC
        {
            type                constant;
            potential           myPotential;
            transportedPhase    airD;
            displacedPhase      waterC;
            C                   0.1;
        }
    }
    \endverbatim

    \table
        Property         | Description        | Required | Default
        potential        | Potential field    | no       | potential
        transportedPhase | Phase transported  | yes      | none
        displacedPhase   | Phase displaced    | yes      | none
        C                | Transport velocity | yes      | none
    \endtable

SourceFiles
    moveAlphaByPotentialModel.C
    moveAlphaByPotentialModelNew.C

\*---------------------------------------------------------------------------*/

#ifndef moveAlphaByPotentialModel_H
#define moveAlphaByPotentialModel_H

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "volFields.H"
#include "dictionary.H"
#include "phaseInterface.H"
#include "runTimeSelectionTables.H"

namespace Foam
{

class phaseSystem;
class phaseModel;

/*---------------------------------------------------------------------------*\
               Class moveAlphaByPotentialModel Declaration
\*---------------------------------------------------------------------------*/

class moveAlphaByPotentialModel
{
    // Private Data

        //- Compressed disperse phase
        const phaseModel& transportedPhase_;

        //- Displaced phase
        const phaseModel& displacedPhase_;

        //- Pointer to potential field
        mutable autoPtr<volScalarField> potentialPtr_;

        //- Potential field name
        const word potentialName_;

        //- Mask field name
        const word maskName_;

protected:

    // Protected Data

        //- Reference to the mesh
        const fvMesh& mesh_;


public:

    //- Runtime type information
    TypeName("moveAlphaByPotentialModel");


    // Declare runtime construction

        declareRunTimeSelectionTable
        (
            autoPtr,
            moveAlphaByPotentialModel,
            dictionary,
            (
                const dictionary& dict,
                const phaseInterface& interface
            ),
            (dict, interface)
        );


    // Static Data Members

        //- Does this model require correcting on fixed flux boundaries?
        static const bool correctFixedFluxBCs = true;


    // Constructors

        //- Construct from a dictionary and an interface
        moveAlphaByPotentialModel
        (
            const dictionary& dict,
            const phaseInterface& interface
        );


    //- Destructor
    virtual ~moveAlphaByPotentialModel();


    // Selectors

        static autoPtr<moveAlphaByPotentialModel> New
        (
            const dictionary& dict,
            const phaseInterface& interface,
            const bool outer=true
        );


    // Member Functions

        //- Coefficient within expression of flux for formation of resolved
        //  structures
        virtual tmp<surfaceScalarField> C() const = 0;

        //- Flux for formation of resolved structures
        virtual tmp<surfaceScalarField> phirc(const phaseModel& phase) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
