/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::curvatureUnderResolution

Description
    The under-resolution indicator function \f$f_\mathrm{UR}\f$ is determined by
    means of the interface curvature with respect to the grid spacing:

    \f[
        f_\mathrm{UR} = \Delta\,\kappa
    \f]

    with interface curvature \f$\kappa\f$ and cell size \f$\Delta\f$. The latter
    is calculated by a delta method, which has to be selected by the user. The
    function is masked, such that it only takes values larger than zero,
    wherever both phases of the pair are present. Subsequently, the indicator
    field is smoothed once.

Usage
    \table
        Property | Description                     | Required  | Default value
        delta    | method for estimating cell size | yes       | none
    \endtable

See also
    Foam::ResolvedPhaseSystem
    Foam::SGSdelta

SourceFiles
    curvatureUnderResolution.C

\*---------------------------------------------------------------------------*/

#ifndef curvatureUnderResolution_H
#define curvatureUnderResolution_H

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "underResolutionModel.H"
#include "SGSdelta.H"

namespace Foam
{
namespace underResolutionModels
{

/*---------------------------------------------------------------------------*\
                   Class curvatureUnderResolution Declaration
\*---------------------------------------------------------------------------*/

class curvatureUnderResolution
:
    public underResolutionModel
{
    // Private Data

        //- Run-time selectable delta model
        autoPtr<Foam::SGSdelta> delta_;


public:

    //- Runtime type information
    TypeName("curvature");


    // Constructors

        //- Construct from a dictionary and an interface
        curvatureUnderResolution
        (
            const dictionary& dict,
            const phaseInterface& interface
        );


    //- Destructor
    virtual ~curvatureUnderResolution();


    // Member Functions

        // Access

            //- Filter width
            inline const volScalarField& delta() const
            {
                return delta_();
            }

        //- Correct under resolution factor
        virtual void correct();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace underResolutionModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
