/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "shearUnderResolution.H"
#include "phaseSystem.H"
#include "fvcGrad.H"
#include "addonFvcSmooth.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace underResolutionModels
{
    defineTypeNameAndDebug(shearUnderResolution, 0);
    addToRunTimeSelectionTable
    (
        underResolutionModel,
        shearUnderResolution,
        dictionary
    );
}
}


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::underResolutionModels::shearUnderResolution::nuI
() const
{
    return
        0.5
       *(interface_.phase1().thermo().nu() + interface_.phase2().thermo().nu());
}


Foam::tmp<Foam::volVectorField>
Foam::underResolutionModels::shearUnderResolution::gradAlphaI() const
{
    volVectorField gradAlphaI(interface_.gradAlpha());
    volScalarField interfaceIndicator(mag(gradAlphaI));

    smoothen(interfaceIndicator, 4);
    const volScalarField infoLocation
    (
        fvc::spreadInterface(gradAlphaI, interfaceIndicator, nLayers_, small)
    );
    smoothen(gradAlphaI);

    return infoLocation*gradAlphaI;
}


void
Foam::underResolutionModels::shearUnderResolution::correctURelativeI()
{
    volScalarField alpha1(interface_.phase1());
    volScalarField alpha2(interface_.phase2());

    volVectorField UI1(interface_.phase1().U());
    volVectorField UI2(interface_.phase2().U());

    smoothen(alpha1);
    smoothen(alpha2);

    const volScalarField infoLocation1
    (
        fvc::spreadInterface(UI1, alpha1, nLayers_, 0.01)
    );

    const volScalarField infoLocation2
    (
        fvc::spreadInterface(UI2, alpha2, nLayers_, 0.01)
    );

    URelativeI_ = infoLocation1*infoLocation2*(UI2 - UI1);
}


Foam::tmp<Foam::volScalarField>
Foam::underResolutionModels::shearUnderResolution::shearInterface
() const
{
    const volVectorField gradAlpha(interface_.gradAlpha());

    tmp<volScalarField> tShearInterface
    (
        new volScalarField
        (
            IOobject
            (
                IOobject::groupName("shearInterface", interface_.name()),
                mesh_.time().name(),
                mesh_
            ),
            mag(URelativeI_ ^ gradAlpha)
           /max
            (
                mag(URelativeI_)*mag(gradAlpha),
                dimensionedScalar(inv(dimTime), small)
            )
        )
    );

    return tShearInterface;
}


Foam::tmp<Foam::volScalarField>
Foam::underResolutionModels::shearUnderResolution::yPlusI
() const
{
    const volVectorField gradAlpha(gradAlphaI());

    const volScalarField invLStarI(sqrt(mag(URelativeI_ ^ gradAlpha)/nuI()));

    tmp<volScalarField> tYPlusI
    (
        new volScalarField
        (
            IOobject
            (
                IOobject::groupName("yPlusI", interface_.name()),
                mesh_.time().name(),
                mesh_
            ),
            invLStarI/max(mag(gradAlpha), interface_.fluid().deltaN())
        )
    );

    return tYPlusI;
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::underResolutionModels::shearUnderResolution::shearUnderResolution
(
    const dictionary& dict,
    const phaseInterface& interface
)
:
    underResolutionModel(dict, interface),

    URelativeI_
    (
        IOobject
        (
            IOobject::groupName("URelativeI", interface.name()),
            mesh_.time().name(),
            mesh_
        ),
        mesh_,
        dimensionedVector(dimVelocity, Zero)
    ),

    nLayers_(dict.lookupOrDefault<label>("nLayers", 5)),

    func_(Function1<scalar>::New("func", dict))
{
    underResolution_.rename
    (
        IOobject::groupName("shearUnderResolution", interface.name())
    );

    correct();
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::underResolutionModels::shearUnderResolution::~shearUnderResolution()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Foam::tmp<Foam::volVectorField>
Foam::underResolutionModels::shearUnderResolution::UrI() const
{
    return tmp<volVectorField>(URelativeI_);
}


void
Foam::underResolutionModels::shearUnderResolution::correct()
{
    correctURelativeI();

    tmp<volScalarField> func
    (
        volScalarField::New("func", mesh_, dimensionedScalar(dimless, Zero))
    );
    func.ref().primitiveFieldRef() = func_->value(yPlusI());

    underResolution_ = shearInterface()*max(min(func, 1.0), Zero);
    smoothen(underResolution_, 1);
}


// ************************************************************************* //
