/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::diameterModels::IATEsource

Description
    IATE (Interfacial Area Transport Equation) bubble diameter model
    run-time selectable sources.

SourceFiles
    IATEsource.C

\*---------------------------------------------------------------------------*/

#ifndef IATEsource_H
#define IATEsource_H

#include "IATE.H"
#include "phaseSystem.H"
#include "mathematicalConstants.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace diameterModels
{

/*---------------------------------------------------------------------------*\
                         Class IATEsource Declaration
\*---------------------------------------------------------------------------*/

class IATEsource
{

protected:

    // Protected data

        //- Reference to the IATE this source applies to
        const IATE& iate_;


public:

    //- Runtime type information
    TypeName("IATEsource");


    // Declare run-time constructor selection table

        declareRunTimeSelectionTable
        (
            autoPtr,
            IATEsource,
            dictionary,
            (
                const IATE& iate,
                const dictionary& dict
            ),
            (iate, dict)
        );


    //- Class used for the read-construction of
    //  PtrLists of IATE sources
    class iNew
    {
        const IATE& iate_;

    public:

        iNew(const IATE& iate)
        :
            iate_(iate)
        {}

        autoPtr<IATEsource> operator()(Istream& is) const
        {
            word type(is);
            dictionary dict(is);
            return IATEsource::New(type, iate_, dict);
        }
    };


    // Constructors

        IATEsource(const IATE& iate)
        :
            iate_(iate)
        {}

        autoPtr<IATEsource> clone() const
        {
            NotImplemented;
            return autoPtr<IATEsource>(nullptr);
        }


    // Selectors

        static autoPtr<IATEsource> New
        (
            const word& type,
            const IATE& iate,
            const dictionary& dict
        );


    //- Destructor
    virtual ~IATEsource()
    {}


    // Member Functions

        const phaseModel& phase() const
        {
            return iate_.phase();
        }

        const phaseSystem& fluid() const
        {
            return iate_.phase().fluid();
        }

        const phaseModel& otherPhase() const
        {
            return fluid().otherPhase(phase());
        }

        scalar phi() const
        {
            return 1.0/(36*constant::mathematical::pi);
        }

        //- Return the bubble surface tension coefficient
        tmp<volScalarField> sigma() const
        {
            return fluid().sigma(phaseInterface(phase(), otherPhase()));
        }

        //- Return the bubble relative velocity
        tmp<volScalarField> Ur() const;

        //- Return the bubble turbulent velocity
        tmp<volScalarField> Ut() const;

        //- Return the bubble Reynolds number
        tmp<volScalarField> Re() const;

        //- Return the bubble drag coefficient
        tmp<volScalarField> CD() const;

        //- Return the bubble Morton number
        tmp<volScalarField> Mo() const;

        //- Return the bubble Eotvos number
        tmp<volScalarField> Eo() const;

        //- Return the bubble turbulent Weber number
        tmp<volScalarField> We() const;

        virtual tmp<fvScalarMatrix> R
        (
            const volScalarField& alphai,
            volScalarField& kappai
        ) const = 0;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace diameterModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
