/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "SecondaryPropertyModel.H"
#include "shapeModel.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class ModelType>
Foam::diameterModels::SecondaryPropertyModel<ModelType>::SecondaryPropertyModel
(
    const dictionary& dict,
    const sizeGroup& group
)
:
    ModelType(dict, group),
    regIOobject
    (
        IOobject
        (
            IOobject::groupName(typeid(ModelType).name(), group.name()),
            group.time().constant(),
            group.mesh()
        )
    ),
    sizeGroup_(group),
    SecondaryPropertyModelTable_()
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

template<class ModelType>
Foam::diameterModels::SecondaryPropertyModel<ModelType>::
~SecondaryPropertyModel()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

template<class ModelType>
const typename Foam::diameterModels::SecondaryPropertyModel<ModelType>::SpTable&
Foam::diameterModels::SecondaryPropertyModel<ModelType>::
SecondaryPropertyModelTable()
{
    if (SecondaryPropertyModelTable_.empty())
    {
        SecondaryPropertyModelTable_ =
            sizeGroup_.mesh().template lookupClass
            <
                SecondaryPropertyModel<ModelType>
            >();
    }

    return SecondaryPropertyModelTable_;
}


template<class ModelType>
const Foam::word Foam::diameterModels::SecondaryPropertyModel<ModelType>::
SecondaryPropertyName
(
    const sizeGroup& fi
) const
{
    return word(IOobject::groupName(typeid(ModelType).name(), fi.name()));
}


template<class ModelType>
void Foam::diameterModels::SecondaryPropertyModel<ModelType>::addCoalescence
(
    const volScalarField& Su,
    const sizeGroup& fj,
    const sizeGroup& fk
)
{
    const volScalarField& propj =
        SecondaryPropertyModelTable()[SecondaryPropertyName(fj)]->fld();

    const volScalarField& propk =
        SecondaryPropertyModelTable()[SecondaryPropertyName(fk)]->fld();

    src() += (propj*fj.x() + propk*fk.x())/(fj.x() + fk.x())*Su;
}


template<class ModelType>
void Foam::diameterModels::SecondaryPropertyModel<ModelType>::addBreakup
(
    const volScalarField& Su,
    const sizeGroup& fj
)
{
    const volScalarField& propj =
        SecondaryPropertyModelTable()[SecondaryPropertyName(fj)]->fld();

    src() += propj*Su;
}


template<class ModelType>
void Foam::diameterModels::SecondaryPropertyModel<ModelType>::addDrift
(
    const volScalarField& Su,
    const sizeGroup& fu,
    const driftModel& model
)
{
    const volScalarField& propu =
        SecondaryPropertyModelTable()[SecondaryPropertyName(fu)]->fld();

    src() += propu*Su;
}


template<class ModelType>
void Foam::diameterModels::SecondaryPropertyModel<ModelType>::addNucleation
(
    const volScalarField& Su,
    const sizeGroup& fi,
    const nucleationModel& model
)
{
    const volScalarField& prop =
        SecondaryPropertyModelTable()[SecondaryPropertyName(fi)]->fld();

    src() += prop*Su;
}


template<class ModelType>
void Foam::diameterModels::SecondaryPropertyModel<ModelType>::reset()
{
    src() = Zero;
}


template<class ModelType>
bool Foam::diameterModels::SecondaryPropertyModel<ModelType>::
writeData(Ostream& os) const
{
    return os.good();
}


// ************************************************************************* //
