/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::diameterModels::SecondaryPropertyModel

Description
    Base class for modeling evolution of secondary representative properties of
    a size class. By default, transport between size classes due to
    coalescence, breakup and drift conserve the property. This behaviour can be
    overridden in a derived class.

See also
    Foam::diameterModels::sizeGroup
    Foam::diameterModels::populationBalanceModel

SourceFiles
    SecondaryPropertyModel.C

\*---------------------------------------------------------------------------*/

#ifndef SecondaryPropertyModel_H
#define SecondaryPropertyModel_H

#include "sizeGroup.H"
#include "driftModel.H"
#include "nucleationModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace diameterModels
{

/*---------------------------------------------------------------------------*\
                   Class SecondaryPropertyModel Declaration
\*---------------------------------------------------------------------------*/

template<class ModelType>
class SecondaryPropertyModel
:
    public ModelType,
    public regIOobject
{

protected:

    // Protected typedefs

        using SpTable = HashTable<const SecondaryPropertyModel<ModelType>*>;

    // Protected Data

        //- Reference to sizeGroup
        const sizeGroup& sizeGroup_;

        //- Table with pointers to all secondary properties of ModelType
        //  available in the registry
        SpTable SecondaryPropertyModelTable_;


public:

    // Constructors

        //- Construct from dictionary and sizeGroup
        SecondaryPropertyModel
        (
            const dictionary& dict,
            const sizeGroup& group
        );

        //- Disallow default bitwise copy construction
        SecondaryPropertyModel
        (
            const SecondaryPropertyModel<ModelType>&
        ) = delete;


    //- Destructor
    virtual ~SecondaryPropertyModel();


    // Member Functions

        // Access

            //- Return table with pointers to all secondary properties of
            //  ModelType available in the registry
            const SpTable& SecondaryPropertyModelTable();

            //- Return name of the secondary property of another class for
            //  for lookup in registry
            const word SecondaryPropertyName(const sizeGroup& fi) const;

            //- Return reference to secondary property field
            virtual const volScalarField& fld() const = 0;

            //- Access to secondary property source
            virtual volScalarField& src() = 0;

        // Edit

            //- Add coalescence contribution to secondary property source
            virtual void addCoalescence
            (
                const volScalarField& Su,
                const sizeGroup& fj,
                const sizeGroup& fk
            );

            //- Add breakup contribution to secondary property source
            virtual void addBreakup
            (
                const volScalarField& Su,
                const sizeGroup& fj
            );

            //- Add drift contribution to secondary property source
            virtual void addDrift
            (
                const volScalarField& Su,
                const sizeGroup& fu,
                const driftModel& model
            );

            //- Add drift contribution to secondary property source
            virtual void addNucleation
            (
                const volScalarField& Su,
                const sizeGroup& fi,
                const nucleationModel& model
            );

            //- Reset secondary property source
            virtual void reset();

        // Write

            //- Dummy write for regIOobject
            bool writeData(Ostream&) const;


    // Member Operators

        //- Disallow default bitwise assignment
        void operator=(const SecondaryPropertyModel<ModelType>&) = delete;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace diameterModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
    #include "SecondaryPropertyModel.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
