/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::diameterModels::shapeModels::fractal

Description
    Class for modelling the shape of particle aggregates using the concept of
    fractal geometry. Returns a collisional diameter

    \f[
        d_{c_i} =
            \frac{6}{\kappa_i}
            \left(
                \frac{v_i \kappa_i^3}{36 \pi \alpha_c}
            \right)^{1/D_{f_i}}\,,
    \f]

    computed from a constant fractal dimension \f$D_{f_i}\f$ and a
    field-dependent surface area to volume ratio \f$\kappa_i\f$, assuming that
    the primary particles in an aggregate have the same size.

    The effect of sintering, i.e. the coalescence of primary particles
    resulting in a loss of surface area, is taken into account by a separate
    source term in the transport equation for \f$\kappa_i\f$.

Usage
    \table
        Property     | Description             | Required    | Default value
        kappa        | Initial and boundary condition value\\
                                               | yes         |
        Df           | Fractal dimension       | yes         |
        alphaC       | Scaling prefactor       | yes         |
    \endtable

See also
    Foam::diameterModels::shapeModel

SourceFiles
    fractal.C

\*---------------------------------------------------------------------------*/

#ifndef fractal_H
#define fractal_H

#include "SecondaryPropertyModel.H"
#include "shapeModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace diameterModels
{
namespace shapeModels
{

class sinteringModel;

/*---------------------------------------------------------------------------*\
                           Class fractal Declaration
\*---------------------------------------------------------------------------*/

class fractal
:
    public SecondaryPropertyModel<shapeModel>
{
public:

    // Public Data types

        //- Surface growth type enumeration
        enum surfaceGrowthTypes
        {
            sgHardSphere,
            sgParkRogak,
            sgConserved
        };

        //- Surface growth type names
        static const NamedEnum<surfaceGrowthTypes, 3> sgTypeNames_;


private:

    // Private Data

        //- Ratio of surface area to volume
        volScalarField kappa_;

        //- Fractal dimension
        dimensionedScalar Df_;

        //- Scaling prefactor
        dimensionedScalar alphaC_;

        //- Collisional diameter
        volScalarField dColl_;

        //- Explicit source
        volScalarField Su_;

        //- Sintering model
        autoPtr<sinteringModel> sinteringModel_;


    // Private Member Functions

        tmp<volScalarField> dColl() const;


public:

    //- Runtime type information
    TypeName("fractal");


    // Constructors

        //- Construct from dictionary and sizeGroup
        fractal
        (
            const dictionary& dict,
            const sizeGroup& group
        );

        //- Disallow default bitwise copy construction
        fractal(const fractal&) = delete;


    //- Destructor
    virtual ~fractal();


    // Member Functions

        // Access

            //- Return reference to secondary property field
            virtual const volScalarField& fld() const;

            //- Access to secondary property source
            virtual volScalarField& src();

            //- Return representative surface area of the sizeGroup
            virtual const tmp<volScalarField> a() const;

            //- Return representative diameter of the sizeGroup
            virtual const tmp<volScalarField> d() const
            {
                return dColl_;
            }

        // Edit

            //- Correct the collisional diameter
            virtual void correct();

            //- Add drift contribution to secondary property source
            virtual void addDrift
            (
                const volScalarField& Su,
                const sizeGroup& fu,
                const driftModel& model
            );


    // Member Operators

        //- Disallow default bitwise assignment
        void operator=(const fractal&) = delete;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace shapeModels
} // End namespace diameterModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
