/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::dispersedPhaseInterface

Description
    Class to represent a interface between phases where one phase is considered
    dispersed within the other.

SourceFiles
    dispersedPhaseInterface.C

\*---------------------------------------------------------------------------*/

#ifndef dispersedPhaseInterface_H
#define dispersedPhaseInterface_H

#include "phaseInterface.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                    Class dispersedPhaseInterface Declaration
\*---------------------------------------------------------------------------*/

class dispersedPhaseInterface
:
    virtual public phaseInterface
{
private:

    // Private Data

        //- Dispersed phase
        const phaseModel& dispersed_;


public:

    //- Runtime type information
    TypeName("dispersedPhaseInterface");


    // Constructors

        //- Construct from phases
        dispersedPhaseInterface
        (
            const phaseModel& dispersed,
            const phaseModel& continuous
        );

        //- Construct from fluid and name
        dispersedPhaseInterface
        (
            const phaseSystem& fluid,
            const word& name
        );


    //- Destructor
    virtual ~dispersedPhaseInterface();


    // Static Member Functions

        //- Return the separator that delimits this interface's name
        static word separator()
        {
            return "dispersedIn";
        }


    // Member Functions

        //- Name
        virtual word name() const;


        // Access

            //- Dispersed phase
            const phaseModel& dispersed() const;

            //- Continuous phase
            const phaseModel& continuous() const;


        // Properties

            //- Relative velocity
            tmp<volVectorField> Ur() const;

            //- Reynolds number
            tmp<volScalarField> Re() const;

            //- Prandtl number
            tmp<volScalarField> Pr() const;

            //- Eotvos number
            tmp<volScalarField> Eo() const;

            //- Eotvos number for given diameter
            tmp<volScalarField> Eo(const volScalarField& d) const;

            //- Morton Number
            tmp<volScalarField> Mo() const;

            //- Takahashi Number
            tmp<volScalarField> Ta() const;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~ Begin Code Repository ~~~~~~~~~~~~~~~~~~~~~~~~~//
            //- Stokes Number
            tmp<volScalarField> St
            (
                const dimensionedScalar& CT,
                const dimensionedScalar& CLambda
            )
            const;
            //- Dimensionless Shear Rate
            tmp<volScalarField> Sr(const volScalarField& Re) const;
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~ End Code Repository ~~~~~~~~~~~~~~~~~~~~~~~~~~//

};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
