/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::segregatedPhaseInterface

Description
    Class to represent a interface between phases where the two phases are
    considered to be segregated; that is, separated by a geometrically complex
    interface for which dispersed representations are inappropriate.

SourceFiles
    segregatedPhaseInterface.C

\*---------------------------------------------------------------------------*/

#ifndef segregatedPhaseInterface_H
#define segregatedPhaseInterface_H

#include "phaseInterface.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                   Class segregatedPhaseInterface Declaration
\*---------------------------------------------------------------------------*/

class segregatedPhaseInterface
:
    virtual public phaseInterface
{
public:

    //- Runtime type information
    TypeName("segregatedPhaseInterface");


    // Constructors

        //- Construct from phases
        segregatedPhaseInterface
        (
            const phaseModel& phase1,
            const phaseModel& phase2
        );

        //- Construct from fluid and name
        segregatedPhaseInterface
        (
            const phaseSystem& fluid,
            const word& name
        );


    //- Destructor
    virtual ~segregatedPhaseInterface();


    // Static Member Functions

        //- Return the separator that delimits this interface's name
        static word separator()
        {
            return "segregatedWith";
        }


    // Member Functions

        //- Name
        virtual word name() const;


//~~~~~~~~~~~~~~~~~~~~~~~~~~~ Begin Code Repository ~~~~~~~~~~~~~~~~~~~~~~~~~//
    // Properties

        //- Interfacial area density
        tmp<volScalarField> Ai() const;

        //- Interface gradient between two phases
        tmp<volVectorField> gradAlpha() const;

        //- Face interface gradient between two phases
        tmp<surfaceVectorField> gradAlphafv() const;

        //- Interface gradient flux between two phases
        tmp<surfaceScalarField> gradAlphaf() const;

        //- Normal to interface between two phases
        tmp<volVectorField> nHat() const;

        //- Normal to interface between two phases
        tmp<surfaceVectorField> nHatfv() const;

        //- Normal to interface between two phases dotted with face areas
        tmp<surfaceScalarField> nHatf() const;

        //- Relative velocity across the interface
        tmp<volVectorField> UrI() const;

        //- Degree of under resolution due to shear or curvature
        tmp<volScalarField> underResolution() const;

        //- Interface curvature
        tmp<volScalarField> K() const;
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~ End Code Repository ~~~~~~~~~~~~~~~~~~~~~~~~~~//
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
