/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "CUDALehrMilliesMewes.H"
#include "OFMemoryManagedField.H"
#include "addToRunTimeSelectionTable.H"
#include <vector>

using namespace Foam::diameterModels::cudaUtilities;

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace diameterModels
{
namespace binaryBreakupModels
{
    defineTypeNameAndDebug(CUDALehrMilliesMewes, 0);
    addToRunTimeSelectionTable
    (
        binaryBreakupModel,
        CUDALehrMilliesMewes,
        dictionary
    );
}
}
}

using Foam::constant::mathematical::pi;

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::diameterModels::binaryBreakupModels::CUDALehrMilliesMewes::
CUDALehrMilliesMewes
(
    const populationBalanceModel& popBal,
    const dictionary& dict
)
:
    LehrMilliesMewes(popBal, dict),
    numberOfStreams_(dict.lookupOrDefault("numberOfStreams", 64)),
    cudaModelObject(numberOfStreams_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::diameterModels::binaryBreakupModels::CUDALehrMilliesMewes::
precompute()
{

    int domainSize = popBal_.alphas().size();
    int numberOfGroups = popBal_.sizeGroups().size();

    // Memory allocation and management
    OFMemoryManagedField alphaPtr =
        cudaPopBal_.setAlphas(domainSize);

    OFMemoryManagedField popBalContinuousTurbulenceEpsilonPtr =
        cudaPopBal_.setContinuousTurbulenceEpsilon(domainSize);

    OFMemoryManagedField sizeGroupsdSphPtr =
        cudaPopBal_.setSizeGroupsdSph(numberOfGroups);

    OFMemoryManagedField popBalContinuousPhaseRhoPtr =
        cudaPopBal_.setContinuousPhaseRho(domainSize);

    OFMemoryManagedField popBalSigmaWithContinuousPhaseIPtr =
        cudaPopBal_.setSigmaWithContinuousPhaseI
        (
            numberOfGroups,
            domainSize
        );

    // Queue pairs to be executed
    std::vector<std::pair<int, int>> pairs;
    List<Pair<label>> bbPairs(popBal_.binaryBreakupPairs());
    pairs.reserve(popBal_.nCorr()*bbPairs.size());
    for (int corr = 0; corr < popBal_.nCorr(); corr++)
    {
        forAll(bbPairs, hh)
        {
            label i(bbPairs[hh].first());
            label j(bbPairs[hh].second());
            pairs.push_back(make_pair(j, i));
        }
    }

    // Memory copy
    forAll(popBal_.alphas(), i)
    {
        alphaPtr[i] = popBal_.alphas()[i];
    }

    volScalarField epsilon(popBal_.continuousTurbulence().epsilon());
    forAll(epsilon, i)
    {
        popBalContinuousTurbulenceEpsilonPtr[i] = epsilon[i];
    }

    forAll(popBal_.sizeGroups(), i)
    {
        const sizeGroup& fi = popBal_.sizeGroups()[i];
        sizeGroupsdSphPtr[i] = fi.dSph().value();
    }

    volScalarField rho (popBal_.continuousPhase().rho());
    forAll(rho, i)
    {
        popBalContinuousPhaseRhoPtr[i] = rho[i];
    }

    forAll(popBal_.sizeGroups(), i)
    {
        const sizeGroup& fi = popBal_.sizeGroups()[i];
        scalarField phaseI = popBal_.sigmaWithContinuousPhase(fi.phase())();
        forAll(phaseI, j)
        {
            popBalSigmaWithContinuousPhaseIPtr[i*domainSize + j] = phaseI[j];
        }
    }

    cudaModelObject.setModelVariables
    (
        SMALL
    );

    // Starting execution
    cudaModelObject.calculateBinaryBreakupRate
    (
        cudaPopBal_,
        domainSize,
        pairs
    );
}


void Foam::diameterModels::binaryBreakupModels::CUDALehrMilliesMewes::
addToBinaryBreakupRate
(
    volScalarField& binaryBreakupRate,
    const label i,
    const label j
)
{
    int domainSize = popBal_.alphas().size();
    OFMemoryManagedField cudaBinaryBreakupRate =
        cudaModelObject.getBinaryBreakupRate(i, j);
    for (int h = 0 ; h < domainSize; h++)
    {
        binaryBreakupRate[h] += cudaBinaryBreakupRate[h];
    }
}


// ************************************************************************* //
