/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::diameterModels::binaryBreakupModels::CUDALehrMilliesMewes

Description
    CUDA implementation of the breakup model of Lehr et al. (2002).

    References:
    \verbatim
        Lehr, F., Millies, M., & Mewes, D. (2002).
        Bubble‐size distributions and flow fields in bubble columns.
        AIChE Journal, 48(11), 2426-2443.
        Eq. 12-16, p. 2429-2430.
    \endverbatim

See also
    Foam::diameterModels::binaryBreakupModels::LehrMilliesMewes

Usage
    \table
        Property         | Description             | Required    | Default value
        numberOfStreams  | Number of CUDA streams  | no          | 64
    \endtable

SourceFiles
    CUDALehrMilliesMewes.C

\*---------------------------------------------------------------------------*/

#ifndef CUDALehrMilliesMewes_H
#define CUDALehrMilliesMewes_H

#include "LehrMilliesMewes.H"
#include "CUDAPopulationBalance.H"
#include "CUDALehrMilliesMewesSupportModel.H"

using namespace Foam::diameterModels::cudaUtilities;

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace diameterModels
{
namespace binaryBreakupModels
{

/*---------------------------------------------------------------------------*\
                    Class CUDALehrMilliesMewes Declaration
\*---------------------------------------------------------------------------*/

class CUDALehrMilliesMewes
:
    public LehrMilliesMewes
{
    // Private Data

        //- Number of CUDA streams
        label numberOfStreams_;

        //- CUDA interface
        CUDALehrMilliesMewesSupportModel cudaModelObject;

        CUDAPopulationBalance cudaPopBal_;


public:

    //- Runtime type information
    TypeName("CUDALehrMilliesMewes");

    // Constructor

        CUDALehrMilliesMewes
        (
            const populationBalanceModel& popBal,
            const dictionary& dict
        );


    //- Destructor
    virtual ~CUDALehrMilliesMewes()
    {}


    // Member Functions

        //- Precompute diameter independent expressions
        virtual void precompute();

        //- Add to binary breakupRate
        virtual void addToBinaryBreakupRate
        (
            volScalarField& binaryBreakupRate,
            const label i,
            const label j
        );
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace binaryBreakupModels
} // End namespace diameterModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
