/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::diameterModels::coalescenceModels::CUDALiaoCoalescenceSupportModel

Description
    CUDA interface for the CUDALiao breakup model.

SourceFiles
    CUDALiaoCoalescenceSupportModel.C

\*---------------------------------------------------------------------------*/

#ifndef CUDALiaoCoalescenceSupportModel_H
#define CUDALiaoCoalescenceSupportModel_H

#include <memory>
#include <vector>
#include "OFMemoryManagedField.H"
#include "CUDAPopulationBalance.H"

using namespace Foam::diameterModels::cudaUtilities;

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace diameterModels
{
namespace coalescenceModels
{

/*---------------------------------------------------------------------------*\
               Class CUDALiaoCoalescenceSupportModel Declaration
\*---------------------------------------------------------------------------*/

class CUDALiaoCoalescenceSupportModel
{
public:
    // Public forward declarations
    class CUDALiaoCoalescenceRunnerPimpl;

    struct CUDALiaoCoalescenceModelPimpl;

    // Constructor
    CUDALiaoCoalescenceSupportModel(int numStreams);

    //- Destructor
    ~CUDALiaoCoalescenceSupportModel();

    // Member Functions
        //- Calculate binary breakupRate
        void calculateCoalescenceRate
        (
            CUDAPopulationBalance popBal,
            int currentDomainSize,
            const std::vector<std::pair<int, int>> &pairs
        );

        //- Return CUDA managed fields
        OFMemoryManagedField getKolmogorovLengthScale
        (
            int domainSize
        );

        OFMemoryManagedField getShearStrainRate
        (
            int domainSize
        );

        OFMemoryManagedField getEddyStrainRate
        (
            int domainSize
        );

        OFMemoryManagedField getUTerminal
        (
            int numberOfGroups
        );

        OFMemoryManagedField getCd
        (
            int numberOfGroups
        );

        OFMemoryManagedField getdCrit
        (
            int domainSize
        );

        OFMemoryManagedField getCPack
        (
            int domainSize
        );

        //- Set values required by the model
        void setModelVariables
        (
            double PMax_,
            double AH_,
            double CEff_,
            double CTurb_,
            double CBuoy_,
            double CShear_,
            double CEddy_,
            double CWake_,
            bool turbulence_,
            bool buoyancy_,
            bool laminarShear_,
            bool eddyCapture_,
            bool wakeEntrainment_,
            double CPackMax_
        );

        //- Get a field with results
        OFMemoryManagedField getCoalescenceRate
        (
            int i,
            int j
        );

private:
        //- Pimpl pattern implementation
        std::unique_ptr<CUDALiaoCoalescenceRunnerPimpl> pimpl;

};

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace coalescenceModels
} // End namespace diameterModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
