/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "structuralModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace LESModels
{

// * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * * //

template<class BasicMomentumTransportModel>
void structuralModel<BasicMomentumTransportModel>::correctNuSgs()
{
    nuSgs_ =
        Ck()*this->delta()
       *sqrt(posPart(structuralModel<BasicMomentumTransportModel>::k()));
    nuSgs_.correctBoundaryConditions();
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class BasicMomentumTransportModel>
structuralModel<BasicMomentumTransportModel>::structuralModel
(
    const alphaField& alpha,
    const rhoField& rho,
    const volVectorField& U,
    const surfaceScalarField& alphaRhoPhi,
    const surfaceScalarField& phi,
    const viscosity& viscosity,
    const word& type
)
:
    BasicMomentumTransportModel
    (
        alpha,
        rho,
        U,
        alphaRhoPhi,
        phi,
        viscosity,
        type
    ),

    nuSgs_
    (
        IOobject
        (
           this->groupName("nuSgs"),
           this->runTime_.name(),
           this->mesh_,
           IOobject::NO_READ,
           IOobject::NO_WRITE
        ),
        this->mesh_,
        dimensionedScalar(dimViscosity, Zero)
    )
{}


template<class BasicMomentumTransportModel>
structuralModel<BasicMomentumTransportModel>::structuralModel
(
    const word& type,
    const alphaField& alpha,
    const rhoField& rho,
    const volVectorField& U,
    const surfaceScalarField& alphaRhoPhi,
    const surfaceScalarField& phi,
    const viscosity& viscosity
)
:
    BasicMomentumTransportModel
    (
        type,
        alpha,
        rho,
        U,
        alphaRhoPhi,
        phi,
        viscosity
    ),

    nuSgs_
    (
        IOobject
        (
           this->groupName("nuSgs"),
           this->runTime_.name(),
           this->mesh_,
           IOobject::NO_READ,
           IOobject::NO_WRITE
        ),
        this->mesh_,
        dimensionedScalar(dimViscosity, Zero)
    )
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

template<class BasicMomentumTransportModel>
tmp<volScalarField> structuralModel<BasicMomentumTransportModel>::nut
() const
{
    return volScalarField::New
    (
        this->groupName("nut"),
        this->mesh_,
        dimensionedScalar(dimViscosity, Zero)
    );
}


template<class BasicMomentumTransportModel>
tmp<scalarField> structuralModel<BasicMomentumTransportModel>::nut
(
    const label patchi
) const
{
    return tmp<scalarField>
    (
        new scalarField(this->mesh_.boundary()[patchi].size(), 0)
    );
}


template<class BasicMomentumTransportModel>
tmp<volScalarField>
structuralModel<BasicMomentumTransportModel>::k() const
{
    return 0.5*tr(structuralModel<BasicMomentumTransportModel>::sigma());
}


template<class BasicMomentumTransportModel>
tmp<volScalarField>
structuralModel<BasicMomentumTransportModel>::epsilon() const
{
    volSymmTensorField D(symm(fvc::grad(this->U())));

    return - structuralModel<BasicMomentumTransportModel>::sigma() && D;
}


template<class BasicMomentumTransportModel>
tmp<volScalarField>
structuralModel<BasicMomentumTransportModel>::omega() const
{
    const dimensionedScalar Ck(dimless, 0.094);

    return epsilon()/(Ck*k());
}


template<class BasicMomentumTransportModel>
tmp<volSymmTensorField>
structuralModel<BasicMomentumTransportModel>::sigma() const
{
    return this->B(this->rho_, this->U())/(this->rho_*max(this->alpha_, small));
}


template<class BasicMomentumTransportModel>
tmp<volSymmTensorField>
structuralModel<BasicMomentumTransportModel>::devTau() const
{
    return this->B(this->rho_, this->U());
}


template<class BasicMomentumTransportModel>
tmp<volSymmTensorField>
structuralModel<BasicMomentumTransportModel>::devTau
(
    const volScalarField& rho,
    const volVectorField& U
) const
{
    return this->B(rho, U);
}


template<class BasicMomentumTransportModel>
tmp<fvVectorMatrix>
structuralModel<BasicMomentumTransportModel>::divDevTau
(
    volVectorField& U
) const
{
    return fvm::Su(fvc::div(this->B(this->rho_, U)), U);
}


template<class BasicMomentumTransportModel>
tmp<fvVectorMatrix>
structuralModel<BasicMomentumTransportModel>::divDevTau
(
    const volScalarField& rho,
    volVectorField& U
) const
{
    return fvm::Su(fvc::div(this->B(rho, U)), U);
}


template<class BasicMomentumTransportModel>
void structuralModel<BasicMomentumTransportModel>::validate()
{
    correctNuSgs();
}


template<class BasicMomentumTransportModel>
void structuralModel<BasicMomentumTransportModel>::correct()
{
    BasicMomentumTransportModel::correct();
    correctNuSgs();

    if (momentumTransportModel::debug && this->runTime_.writeTime())
    {
        Info<< "Writing " << this->nuSgs_.name() << endl;
        this->nuSgs_.write();
    }
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace LESModels
} // End namespace Foam

// ************************************************************************* //
