/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::LESModels::structuralModel

Description
    Abstract base class for structural LES models.

SourceFiles
    structuralModel.C

\*---------------------------------------------------------------------------*/

#ifndef structuralModel_H
#define structuralModel_H

#include "LESModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace LESModels
{

/*---------------------------------------------------------------------------*\
                       Class structuralModel Declaration
\*---------------------------------------------------------------------------*/

template<class BasicMomentumTransportModel>
class structuralModel
:
    public BasicMomentumTransportModel
{
public:

    typedef typename BasicMomentumTransportModel::alphaField alphaField;
    typedef typename BasicMomentumTransportModel::rhoField rhoField;


protected:

    // Protected data

        //- Resulting Eddy-Viscosity due to sub-grid stress tensor of structural
        //  model, only for post processing purposes
        volScalarField nuSgs_;


private:

    // Private Member Functions

        // Disallow default bitwise copy construct and assignment
        structuralModel(const structuralModel&);
        void operator=(const structuralModel&);

        //- Correct eddy-viscosity due to the present model
        void correctNuSgs();

        //- Return model coefficient
        virtual dimensionedScalar Ck() const = 0;


protected:

        //- Return the sub-grid stress tensor
        virtual tmp<volSymmTensorField> B
        (
            const rhoField& rho,
            const volVectorField& U
        ) const = 0;


public:

    // Constructors

        //- Construct from components, inherit from another turbulence model
        structuralModel
        (
            const alphaField& alpha,
            const rhoField& rho,
            const volVectorField& U,
            const surfaceScalarField& alphaRhoRhi,
            const surfaceScalarField& phi,
            const viscosity& viscosity,
            const word& type
        );

        //- Construct from components, inherit from abstract model class
        structuralModel
        (
            const word& type,
            const alphaField& alpha,
            const rhoField& rho,
            const volVectorField& U,
            const surfaceScalarField& alphaRhoRhi,
            const surfaceScalarField& phi,
            const viscosity& viscosity
        );


    //- Destructor
    virtual ~structuralModel()
    {}


    // Member Functions

       //- Return the turbulence viscosity, i.e. 0 for this model
        virtual tmp<volScalarField> nut() const;

        //- Return the turbulence viscosity on patch, i.e. 0 for this model
        virtual tmp<scalarField> nut(const label patchi) const;

        //- Return the SGS turbulent kinetic energy
        virtual tmp<volScalarField> k() const;

        //- Return the SGS turbulent dissipation
        virtual tmp<volScalarField> epsilon() const;

        //- Return the turbulence specific dissipation rate
        virtual tmp<volScalarField> omega() const;

        //- Return the Reynolds stress tensor
        virtual tmp<volSymmTensorField> sigma() const;

        //- Return the effective stress tensor
        virtual tmp<volSymmTensorField> devTau() const;

        //- Return the effective stress tensor
        virtual tmp<volSymmTensorField> devTau
        (
            const volScalarField& rho,
            const volVectorField& U
        ) const;

        //- Return the source term for the momentum equation
        virtual tmp<fvVectorMatrix> divDevTau(volVectorField& U) const;

        //- Return the source term for the momentum equation
        virtual tmp<fvVectorMatrix> divDevTau
        (
            const volScalarField& rho,
            volVectorField& U
        ) const;

        //- Validate the turbulence fields after construction
        //  Update turbulence viscosity and other derived fields as requires
        virtual void validate();

        //- Correct Eddy-Viscosity and related properties
        virtual void correct();

        //- Read model coefficients if they have changed
        virtual bool read()
        {
            return BasicMomentumTransportModel::read();
        }
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace LESModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
    #include "structuralModel.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
