/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "NicoudSigma.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace LESModels
{

// * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * * //

template<class BasicMomentumTransportModel>
tmp<volSymmTensorField> NicoudSigma<BasicMomentumTransportModel>::G
(
    const tmp<volTensorField>& gradU
) const
{
    return symm(gradU() & gradU().T());
}


template<class BasicMomentumTransportModel>
tmp<volScalarField> NicoudSigma<BasicMomentumTransportModel>::Dsigma
(
    const tmp<volSymmTensorField>& tG
) const
{
    volVectorField sigmaSqr(eigenValues(tG()));

    volScalarField sigma3(sqrt(posPart(sigmaSqr.component(vector::X))));
    volScalarField sigma2(sqrt(posPart(sigmaSqr.component(vector::Y))));
    volScalarField sigma1(sqrt(posPart(sigmaSqr.component(vector::Z))));

    return tmp<volScalarField>
    (
        sigma3*(sigma1 - sigma2)*(sigma2 - sigma3)
       /max
        (
            sigmaSqr.component(vector::Z),
            dimensionedScalar(dimensionSet(0, 0, -2, 0, 0), small)
        )
    );
}


// * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * * //

template<class BasicMomentumTransportModel>
void NicoudSigma<BasicMomentumTransportModel>::correctNut()
{
    this->nut_ = sqr(Csigma_*this->delta())*Dsigma();
    this->nut_.correctBoundaryConditions();
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class BasicMomentumTransportModel>
NicoudSigma<BasicMomentumTransportModel>::NicoudSigma
(
    const alphaField& alpha,
    const rhoField& rho,
    const volVectorField& U,
    const surfaceScalarField& alphaRhoPhi,
    const surfaceScalarField& phi,
    const viscosity& viscosity,
    const word& type
)
:
    LESeddyViscosity<BasicMomentumTransportModel>
    (
        type,
        alpha,
        rho,
        U,
        alphaRhoPhi,
        phi,
        viscosity
    ),

    Csigma_
    (
        dimensioned<scalar>::lookupOrAddToDict
        (
            "Csigma",
            this->coeffDict_,
            1.5
        )
    )
{
    if (type == typeName)
    {
        this->printCoeffs(type);
    }
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

template<class BasicMomentumTransportModel>
tmp<volScalarField> NicoudSigma<BasicMomentumTransportModel>::k() const
{
    return volScalarField::New
    (
        this->groupName("k"),
        sqr(1/this->Ck_*this->nut()/this->delta())
    );
}


template<class BasicMomentumTransportModel>
tmp<volScalarField> NicoudSigma<BasicMomentumTransportModel>::epsilon() const
{
    volSymmTensorField D(symm(fvc::grad(this->U())));

    return volScalarField::New
    (
        this->groupName("epsilon"),
        2*this->nut_*magSqr(D)
    );
}


template<class BasicMomentumTransportModel>
tmp<volScalarField> NicoudSigma<BasicMomentumTransportModel>::omega() const
{
    return volScalarField::New
    (
        this->groupName("omega"),
        epsilon()/(this->Ck_*k())
    );
}


template<class BasicMomentumTransportModel>
void NicoudSigma<BasicMomentumTransportModel>::correct()
{
    LESeddyViscosity<BasicMomentumTransportModel>::correct();
    correctNut();

    if (momentumTransportModel::debug && this->runTime_.writeTime())
    {
        tmp<volSymmTensorField> tauSgs
        (
            volSymmTensorField::New
            (
                this->groupName("tauSgs"),
               -this->alpha_*this->rho_*this->nut()
               *dev(twoSymm(fvc::grad(this->U_)))
            )
        );
        Info<< "Writing " << tauSgs->name() << endl;
        tauSgs().write();

        Info<< "Writing " << this->nut_.name() << endl;
        this->nut_.write();
    }
}


template<class BasicMomentumTransportModel>
bool NicoudSigma<BasicMomentumTransportModel>::read()
{
    if (LESeddyViscosity<BasicMomentumTransportModel>::read())
    {
        Csigma_.readIfPresent(this->coeffDict());

        return true;
    }
    else
    {
        return false;
    }
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace LESModels
} // End namespace Foam

// ************************************************************************* //
