/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::epsilonScalableWallFunctionFvPatchScalarField

Description
    This boundary condition provides a turbulence dissipation wall function
    condition for low- and high-Reynolds number turbulent flow cases.

    The condition can be applied to wall boundaries for which it inserts near
    wall epsilon values directly into the epsilon equation to act as a
    constraint.

    The model operates in two modes, based on the computed laminar-to-turbulent
    switch-over y+ value derived from kappa and E.

Usage
    \table
        Property     | Description             | Required    | Default value
        Cmu          | model coefficient       | no          | 0.09
        kappa        | Von Karman constant     | no          | 0.41
        E            | model coefficient       | no          | 9.8
    \endtable

    Example of the boundary condition specification:
    \verbatim
    <patchName>
    {
        type            epsilonScalableWallFunction;
    }
    \endverbatim

SeeAlso
    Foam::epsilonWallFunctionFvPatchScalarField

SourceFiles
    epsilonScalableWallFunctionFvPatchScalarField.C

\*---------------------------------------------------------------------------*/

#ifndef epsilonScalableWallFunctionFvPatchScalarField_H
#define epsilonScalableWallFunctionFvPatchScalarField_H

#include "epsilonWallFunctionFvPatchScalarField.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
        Class epsilonScalableWallFunctionFvPatchScalarField Declaration
\*---------------------------------------------------------------------------*/

class epsilonScalableWallFunctionFvPatchScalarField
:
    public epsilonWallFunctionFvPatchScalarField
{
protected:

    // Protected Member Functions

        //- Calculate the epsilon and G
        virtual void calculate
        (
            const momentumTransportModel& turbulence,
            const List<scalar>& cornerWeights,
            const fvPatch& patch,
            scalarField& G,
            scalarField& epsilon
        );


public:

    //- Runtime type information
    TypeName("epsilonScalableWallFunction");


    // Constructors

        //- Construct from patch and internal field
        epsilonScalableWallFunctionFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct from patch, internal field and dictionary
        epsilonScalableWallFunctionFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping given
        //  epsilonScalableWallFunctionFvPatchScalarField
        //  onto a new patch
        epsilonScalableWallFunctionFvPatchScalarField
        (
            const epsilonScalableWallFunctionFvPatchScalarField&,
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const fvPatchFieldMapper&
        );

        //- Disallow copy without setting internal field reference
        epsilonScalableWallFunctionFvPatchScalarField
        (
            const epsilonScalableWallFunctionFvPatchScalarField&
        ) = delete;

        //- Construct as copy setting internal field reference
        epsilonScalableWallFunctionFvPatchScalarField
        (
            const epsilonScalableWallFunctionFvPatchScalarField&,
            const DimensionedField<scalar, volMesh>&
        );


    //- Destructor
    virtual ~epsilonScalableWallFunctionFvPatchScalarField()
    {}
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
