/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "omegaMenterWallFunctionFvPatchScalarField.H"
#include "nutWallFunctionFvPatchScalarField.H"
#include "momentumTransportModel.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * * //

void omegaMenterWallFunctionFvPatchScalarField::calculate
(
    const momentumTransportModel& turbModel,
    const List<scalar>& cornerWeights,
    const fvPatch& patch,
    scalarField& G,
    scalarField& omega
)
{
    const label patchi = patch.index();

    const nutWallFunctionFvPatchScalarField& nutw =
        nutWallFunctionFvPatchScalarField::nutw(turbModel, patchi);

    const scalarField& y = turbModel.y()[patchi];

    const scalar Cmu25 = pow025(nutw.Cmu());

    const tmp<volScalarField> tk = turbModel.k();
    const volScalarField& k = tk();

    const tmp<scalarField> tnuw = turbModel.nu(patchi);
    const scalarField& nuw = tnuw();

    const fvPatchVectorField& Uw = turbModel.U().boundaryField()[patchi];

    const scalarField magUp(mag(Uw.patchInternalField() - Uw));

    // Set omega and G
    forAll(nutw, facei)
    {
        const label celli = patch.faceCells()[facei];
        const scalar w = cornerWeights[facei];

        const scalar Rey = y[facei]*sqrt(k[celli])/nuw[facei];
        const scalar yStar = Cmu25*Rey;


        const scalar omegaVis = 6*nuw[facei]/(beta1_*sqr(y[facei]));
        const scalar omegaLog = sqrt(k[celli])/(Cmu25*nutw.kappa()*y[facei]);

        omega[celli] += w*sqrt(sqr(omegaVis) + sqr(omegaLog));

        const scalar uTauVis2 = nuw[facei]*magUp[facei]/y[facei];

        scalar uTauLog = 0.0;
        if (yStar > scalar(1.0))
        {
            uTauLog = nutw.kappa()*magUp[facei]/log(yStar*nutw.E());
        }

        const scalar uStarLog = Cmu25*sqrt(k[celli]);

        const scalar uTau = pow025(sqr(uTauVis2) + pow4(uTauLog));
        const scalar uStar = pow025(sqr(uTauVis2) + pow4(uStarLog));

        const scalar uPlus =
            inv(pow025
            (
                pow4(inv(yStar + vSmall))
              + pow4(nutw.kappa()*inv(log(nutw.E()*max(yStar, 1.0))))
            ));

        const scalar gradUplus =
            pow5(uPlus/(yStar + vSmall))
          + pow5
            (
                uPlus*nutw.kappa()*inv(log(nutw.E()*max(yStar, 1.0)))
            )
           /(nutw.kappa()*(yStar + vSmall));

        G[celli] +=
            w
           *sqr(uTau*uStar)/nuw[facei]
           *(1 - gradUplus)*gradUplus;
    }
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

omegaMenterWallFunctionFvPatchScalarField::
omegaMenterWallFunctionFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF
)
:
    omegaWallFunctionFvPatchScalarField(p, iF)
{}


omegaMenterWallFunctionFvPatchScalarField::
omegaMenterWallFunctionFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const dictionary& dict
)
:
    omegaWallFunctionFvPatchScalarField(p, iF, dict)
{}


omegaMenterWallFunctionFvPatchScalarField::
omegaMenterWallFunctionFvPatchScalarField
(
    const omegaMenterWallFunctionFvPatchScalarField& ptf,
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    omegaWallFunctionFvPatchScalarField(ptf, p, iF, mapper)
{}


omegaMenterWallFunctionFvPatchScalarField::
omegaMenterWallFunctionFvPatchScalarField
(
    const omegaMenterWallFunctionFvPatchScalarField& owfpsf,
    const DimensionedField<scalar, volMesh>& iF
)
:
    omegaWallFunctionFvPatchScalarField(owfpsf, iF)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void omegaMenterWallFunctionFvPatchScalarField::write(Ostream& os) const
{
    omegaWallFunctionFvPatchScalarField::write(os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

makePatchTypeField
(
    fvPatchScalarField,
    omegaMenterWallFunctionFvPatchScalarField
);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
