/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "filmThickness.H"
#include "fvMesh.H"
#include "interpolation.H"
#include "meshSearch.H"
#include "lineCellFace.H"
#include "fvc.H"
#include "IOobject.H"
#include "uniformDimensionedFields.H"
#include "volFields.H"
#include "addToRunTimeSelectionTable.H"
#include "Pstream.H"
#include "Time.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    namespace functionObjects
    {
        defineTypeNameAndDebug(filmThickness, 0);
        addToRunTimeSelectionTable(functionObject, filmThickness, dictionary);
    }
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::functionObjects::filmThickness::filmThickness
(
    const word& name,
    const Foam::Time& runTime,
    const Foam::dictionary& dict
)
:
    fvMeshFunctionObject(name, runTime, dict),
    alphaName_("alpha"),
    patchNames_(),
    maxDistance_(1.0),
    interpolationScheme_("cell")
{
    read(dict);

    if (Pstream::nProcs() > 1)
    {
        FatalErrorInFunction << "FunctionObject " << typeName
            << " not intended for parallel execution. Run in serial, e.g.:"
            << nl << nl
            << "  foamPostProcess -func \"filmThickness(patches=(walls), "
            << "alpha=alpha.water, maxDistance=0.1)\"" << nl << nl
            << exit(FatalError);
    }
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::functionObjects::filmThickness::~filmThickness()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

bool Foam::functionObjects::filmThickness::read(const Foam::dictionary& dict)
{
    fvMeshFunctionObject::read(dict);

    dict.readIfPresent("alpha", alphaName_);
    dict.readIfPresent("patches", patchNames_);
    dict.readIfPresent("maxDistance", maxDistance_);
    dict.readIfPresent("interpolationScheme", interpolationScheme_);

    return true;
}


Foam::wordList Foam::functionObjects::filmThickness::fields() const
{
    return Foam::wordList(alphaName_);
}


bool Foam::functionObjects::filmThickness::execute()
{
    return true;
}


bool Foam::functionObjects::filmThickness::write()
{
    volScalarField filmThickness
    (
        IOobject
        (
            "filmThickness",
            mesh_.time().name(),
            mesh_,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        mesh_,
        dimensionedScalar(dimLength, Zero)
    );

    const volScalarField smoothAlpha =
        fvc::average(mesh_.lookupObject<volScalarField>(alphaName_));

    autoPtr<interpolation<scalar>> interpolatorAlpha
    (
        interpolation<scalar>::New(interpolationScheme_, smoothAlpha)
    );

    forAll(patchNames_, patchi)
    {
        const label patchID =
            mesh_.boundaryMesh().findPatchID(patchNames_[patchi]);
        const fvPatch& patchField = mesh_.boundary()[patchID];

        const vectorField nHats = patchField.nf();

        fvPatchScalarField& filmThicknessw =
            filmThickness.boundaryFieldRef()[patchID];

        forAll(patchField.Cf(), li)
        {
            const vector nHat = nHats[li];
            const point& faceCenter = patchField.Cf()[li];

            const sampledSets::lineCellFace set
            (
                "",
                mesh_,
                meshSearch(mesh_),
                coordSet::axisTypeNames_[coordSet::axisType::XYZ],
                faceCenter,
                faceCenter - nHat*maxDistance_
            );

            scalar sumLengthAlpha = 0;
            for (label si = 0; si < set.size() - 1; ++si)
            {
                if (set.segments()[si] != set.segments()[si + 1])
                {
                    continue;
                }

                const vector& p0 = set.pointCoord(si);
                const vector& p1 = set.pointCoord(si + 1);
                const label c0 = set.cells()[si];
                const label c1 = set.cells()[si + 1];
                const label f0 = set.faces()[si];
                const label f1 = set.faces()[si + 1];

                if (f0 != -1 && f1 != -1) continue;

                const scalar a0 = interpolatorAlpha->interpolate(p0, c0, f0);
                const scalar a1 = interpolatorAlpha->interpolate(p1, c1, f1);
                const scalar l = -nHat & (p1 - p0);

                if ((a0 < a1 && a0 < 0.5) || a0 < 0.1 )
                {
                    break;
                }

                sumLengthAlpha += 0.5*l*(a0 + a1);
            }

            filmThicknessw[li] = sumLengthAlpha;
        }
    }

    filmThickness.write();

    return true;
}


// ************************************************************************* //
