/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::functionObjects::filmThickness

Description
    This function objects calculates the thickness of a film attached to a set
    of boundary patches. The results are stored in a volScalarField named
    "filmThickness" and the values are written to the boundary field.
    forward.

    For each cell centre of the specified boundary patch, the distance of the
    interface in the direction normal to the patch is calculated. An integral
    approach is used. The integration is stopped as soon as one of the following
    conditions is met:
        a) A maximum distance from the boundary patch is reached (maxDistance).
        b) Alpha is below 0.5 and increases when increasing the distance from
           the boundary patch (no integration across gaps).
        c) The value of alpha is below 0.1.

    The integration approach used is according to the method proposed by
    Porombka (2023). Condition b) has been modified to be only active for
    alpha < 0.5, in order to ignore small fluctuations inside the film.

    Currently, this function object can only be executed in serial. Therefore,
    computing times might be very long, depending on the size of the considered
    patch fields. It is recommended to run this function object via
    'foamPostProcessing' utility to avoid long simulation times.

    Reference:
    \verbatim
        Porombka, P. (2023).
        Experimental Investigation and Modelling of Annular Flow in Pipes and
        the Prediction of the Liquid Distribution in Compact Heat Exchangers.
        PhD Thesis, TU Dresden.
    \endverbatim

    Example of function object specification:
    \verbatim
    filmThickness1
    {
        type            filmThickness;
        libs            ("libaddonFieldFunctionObjects.so");

        alpha           alpha.water;
        patches         (walls);
        maxDistance     2e-3;
    }
    \endverbatim

Usage
    \table
        Property | Description                | Required   | Default value
        type     | type name: filmThickness   | yes        |
        alpha    | Name of alpha field to process | no     | alpha
        patches  | List of patches to process     | yes    |
        maxDistance | Maximum considered thickness| yes    |
        interpolationScheme| Scheme for interpolation | no | cell
    \endtable

SourceFiles
    filmThickness.C

See also
    Foam::functionObjects::interfaceHeight

\*---------------------------------------------------------------------------*/

#ifndef filmThickness_H
#define filmThickness_H

#include "fvMeshFunctionObject.H"
#include "logFiles.H"
#include "point.H"
#include "volFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace functionObjects
{

/*---------------------------------------------------------------------------*\
                        Class filmThickness Declaration
\*---------------------------------------------------------------------------*/

class filmThickness
:
    public fvMeshFunctionObject
{
    // Private Data

        //- Name of the alpha field
        word alphaName_;

        //- Names of boundary patch(es) to calculate film thickness on
        wordList patchNames_;

        //- Maximum distance from patch
        scalar maxDistance_;

        //- Interpolation scheme for alpha field
        word interpolationScheme_;


public:

    //- Runtime type information
    TypeName("filmThickness");


    // Constructors

        //- Construct from Time and dictionary
        filmThickness
        (
            const word& name,
            const Time& runTime,
            const dictionary& dict
        );


    //- Destructor
    virtual ~filmThickness();


    // Member Functions

        //- Read
        virtual bool read(const dictionary&);

        //- Return the list of fields required
        virtual wordList fields() const;

        //- Execute
        virtual bool execute();

        //- Write
        virtual bool write();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace functionObjects
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
