/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2023 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

Class
    Foam::profiler

Description
    Class which can be used for profiling, i.e. measuring the computational time
    spent for a function and the number of calls to it. The result is printed
    upon destruction, i.e. at the end of the log file.

Usage
    At the top of the source file:

    \verbatim
        #include "profiler.H"

        static std::auto_ptr<profiler> profiling
        (
            new profiler("function")
        );
    \endverbatim

    Before and after the portion of code to be analysed:

    \verbatim
        profiling->proceed();

        function();

        profiling->pause();
    \endverbatim

    In Make/options of the corresponding application or library:

    \verbatim
        EXE_INC = \
            -I$(WM_PROJECT_SITE)/$(WM_PROJECT_VERSION)/src/performance/lnInclude
    \endverbatim

\*---------------------------------------------------------------------------*/

#ifndef profiler_H
#define profiler_H

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include <chrono>
#include <memory>

/*---------------------------------------------------------------------------*\
                          Class profiler Declaration
\*---------------------------------------------------------------------------*/

class profiler
{
    //- Private Data

        //- Name of function to be analysed
        std::string name_;

        //- Start time in microseconds
        std::chrono::time_point<std::chrono::steady_clock> start_;

        //- Accumulated time in microseconds
        std::chrono::microseconds accumulatedTime_;

        //- Accumulated number of calls
        int numberOfCalls_;


public:

    //- Constructor

        //- Construct with name of function to be analysed
        profiler(const std::string& name)
        :
            name_(name),
            accumulatedTime_(0),
            numberOfCalls_(0)
        {}


    //- Destructor
    ~profiler()
    {
        std::cout<< "Required time for " << name_ << " "
                 << accumulatedTime_.count()/1e3 << " ms, number of calls = "
                 << numberOfCalls_ << std::endl;
    }


    // Member Functions

        //- Continue counting
        void proceed()
        {
            ++numberOfCalls_;

            start_ = std::chrono::steady_clock::now();
        }

        //- Pause counting
        void pause()
        {
            auto diff = std::chrono::steady_clock::now() - start_;

            accumulatedTime_ +=
                std::chrono::duration_cast
                <
                    std::chrono::microseconds
                >(diff);
        }
};

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
